package com.ruoyi.common.utils.document;

import com.itextpdf.text.pdf.BaseFont;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.framework.web.domain.server.Sys;
import freemarker.template.Configuration;
import freemarker.template.Template;
import org.xhtmlrenderer.pdf.ITextRenderer;


import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.Locale;
import java.util.Map;

/**
 * @Description: 通过模板导出pdf文件
 * @Author: cq
 * @CreateDate: 2020/9/21 9:37
 */
public class PDFTemplateUtil {

    /**
     * 通过模板导出pdf文件
     *
     * @param map              数据
     * @param templateFileName 模板文件名
     * @throws Exception
     */
    private static ByteArrayOutputStream createPDF(Map<String, Object> map, String templateFileName, String filePath) throws Exception {
        // 创建一个FreeMarker实例, 负责管理FreeMarker模板的Configuration实例
        Configuration cfg = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
        // 指定FreeMarker模板文件的位置
        cfg.setClassForTemplateLoading(PDFTemplateUtil.class, "/freeMaker");
        ITextRenderer renderer = new ITextRenderer();
        OutputStream out = new ByteArrayOutputStream();
        try {
            // 设置 css中 的字体样式（暂时仅支持宋体和黑体） 必须，不然中文不显示
            renderer.getFontResolver().addFont("/freeMaker/font/simsun.ttc", BaseFont.IDENTITY_H, BaseFont.NOT_EMBEDDED);
            // 设置模板的编码格式
            cfg.setEncoding(Locale.CHINA, "UTF-8");
            // 获取模板文件
            Template template = cfg.getTemplate(templateFileName, "UTF-8");
            StringWriter writer = new StringWriter();

            // 将数据输出到html中
            template.process(map, writer);
            writer.flush();

            String html = writer.toString();
            // 把html代码传入渲染器中
            renderer.setDocumentFromString(html);

            // 设置模板中的图片路径 （这里的images在resources/freeMaker目录下） 模板中img标签src路径需要相对路径加图片名 如<img src="images/xh.jpg"/>
            //String url = PDFTemplateUtil.class.getClassLoader().getResource("freeMaker").toURI().toString();
            //renderer.getSharedContext().setBaseURL(url);
            //本地测试路径为（file:/D:/ssyFile/），服务器路径为（file:/ssyFile/）

            System.out.println(filePath);
            renderer.getSharedContext().setBaseURL(filePath + "/");
            renderer.layout();

            renderer.createPDF(out, false);
            renderer.finishPDF();
            out.flush();
            return (ByteArrayOutputStream) out;
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }

    /**
     * 通过模板导出pdf文件
     *
     * @param templateName 模板文件名(模板文件名+后缀)
     * @param pdfName      pdf文件名(pdf文件名+后缀)
     * @param map          数据
     * @param response     HttpServletResponse
     * @throws Exception
     */
    public static void exportPdf(String templateName, String pdfName, String filePath, Map<String, Object> map, HttpServletResponse response) throws Exception {
        ByteArrayOutputStream baos = null;
        OutputStream out = null;
        try {
            baos = createPDF(map, templateName, filePath);
            // 设置响应消息头，告诉浏览器当前响应是一个下载文件
            response.setContentType("application/x-msdownload");
            // 告诉浏览器，当前响应数据要求用户干预保存到文件中，以及文件名是什么 如果文件名有中文，必须URL编码
            String fileName = URLEncoder.encode(pdfName, "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            out = response.getOutputStream();
            baos.writeTo(out);
            baos.close();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("导出失败：" + e.getMessage());
        } finally {
            if (baos != null) {
                baos.close();
            }
            if (out != null) {
                out.close();
            }
        }
    }

    /**
     * 通过模板导出pdf文件
     *
     * @param templateName 模板文件名(模板文件名+后缀)
     * @param pdfName      pdf文件名(pdf文件名+后缀)
     * @param filePath     文件路径
     * @param filePath2    图片路径
     * @param map          数据
     * @param response     HttpServletResponse
     * @throws Exception
     */
    public static void createPdf(String templateName, String pdfName, String filePath, String filePath2, Map<String, Object> map, HttpServletResponse response) throws Exception {
        ByteArrayOutputStream baos = null;
        FileOutputStream fileOutputStream = null;
        try {
            File file = new File(filePath);
            if (!file.isDirectory()) {
                file.mkdirs();
            }
            baos = createPDF(map, templateName, filePath2);
            fileOutputStream = new FileOutputStream(filePath + "/" + pdfName);
            fileOutputStream.write(baos.toByteArray());
            baos.close();
            fileOutputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("导出失败：" + e.getMessage());
        } finally {
            if (baos != null) {
                baos.close();
            }
            if (fileOutputStream != null) {
                fileOutputStream.close();
            }
        }
    }

}
