package com.ruoyi.project.affair.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.affair.domain.TPpe;
import com.ruoyi.project.affair.service.ITPpeService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * PPE发放登记Controller
 *
 * @author ruoyi
 * @date 2020-12-02
 */
@RestController
@RequestMapping("/affair/ppe")
public class TPpeController extends BaseController
{
    @Autowired
    private ITPpeService tPpeService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    /**
     * 查询PPE发放登记列表
     */
    @PreAuthorize("@ss.hasPermi('affair:ppe:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPpe tPpe)
    {
        startPage();
        List<TPpe> list = tPpeService.selectTPpeList(tPpe);
        return getDataTable(list);
    }

    /**
     * 导出PPE发放登记列表
     */
    @PreAuthorize("@ss.hasPermi('affair:ppe:export')")
    @Log(title = "PPE发放登记", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPpe tPpe)
    {
        List<TPpe> list = tPpeService.selectTPpeList(tPpe);
        ExcelUtil<TPpe> util = new ExcelUtil<TPpe>(TPpe.class);
        return util.exportExcel(list, "ppe");
    }

    /**
     * 获取PPE发放登记详细信息
     */
    @PreAuthorize("@ss.hasPermi('affair:ppe:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tPpeService.selectTPpeById(id));
    }

    /**
     * 新增PPE发放登记
     */
    @PreAuthorize("@ss.hasPermi('affair:ppe:add')")
    @Log(title = "PPE发放登记", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPpe tPpe)
    {
        tPpe.setCreaterCode(getUserId().toString());
        return toAjax(tPpeService.insertTPpe(tPpe));
    }

    /**
     * 修改PPE发放登记
     */
    @PreAuthorize("@ss.hasPermi('affair:ppe:edit')")
    @Log(title = "PPE发放登记", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPpe tPpe)
    {
        tPpe.setUpdaterCode(getUserId().toString());
        tPpe.setUpdatedate(new Date());
        return toAjax(tPpeService.updateTPpe(tPpe));
    }

    /**
     * 删除PPE发放登记
     */
    @PreAuthorize("@ss.hasPermi('affair:ppe:remove')")
    @Log(title = "PPE发放登记", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tPpeService.deleteTPpeByIds(ids));
    }

    /**
     * 批量导入PPE发放登记
     */
    @PreAuthorize("@ss.hasPermi('affair:ppe:add')")
    @Log(title = "PPE发放登记", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPpe> list = new ArrayList<TPpe>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPpe entity = new TPpe();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setName(cellValue);//姓名
                    } else if (j == 2) {
                        entity.setPpename(cellValue);//名称
                    } else if (j == 3) {
                        entity.setSpecification(cellValue);//规格型号
                    } else if (j == 4) {
                        if (cellValue != ""){
                            entity.setQuantity(Long.parseLong(cellValue));//数量
                        }
                    } else if (j == 5) {
                        if (cellValue.length() > 3) {
                            entity.setServicetime(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//使用期限
                        }
                    } else if (j == 6) {
                        if (cellValue.length() > 3) {
                            entity.setReceivedate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//领用日期
                        }
                    } else if (j == 7) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 8) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPpe t : list
        ) {
            failNum++;
            try {
                tPpeService.insertTPpe(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
