package com.ruoyi.project.aspen.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.aspen.domain.TElecdashboardDay;
import com.ruoyi.project.aspen.service.ITElecdashboardDayService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 电厂大屏每日数据Controller
 *
 * @author ruoyi
 * @date 2022-09-14
 */
@RestController
@RequestMapping("/aspen/day")
public class TElecdashboardDayController extends BaseController
{
    @Autowired
    private ITElecdashboardDayService tElecdashboardDayService;

    /**
     * 查询日期最靠前的30条电厂大屏每日数据
     */
    @GetMapping("/listLatest30")
    public TableDataInfo listLatest30()
    {
        startPage();
        List<TElecdashboardDay> list = tElecdashboardDayService.selectTElecdashboardDayListLatest30();
        return getDataTable(list);
    }

    /**
     * 查询日期最靠前的7条电厂大屏每日数据
     */
    @GetMapping("/listLatest7")
    public TableDataInfo listLatest7()
    {
        startPage();
        List<TElecdashboardDay> list = tElecdashboardDayService.selectTElecdashboardDayListLatest7();
        return getDataTable(list);
    }

    /**
     * 查询电厂大屏每日数据列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TElecdashboardDay tElecdashboardDay)
    {
        startPage();
        List<TElecdashboardDay> list = tElecdashboardDayService.selectTElecdashboardDayList(tElecdashboardDay);
        return getDataTable(list);
    }

    /**
     * 导出电厂大屏每日数据列表
     */
    @Log(title = "电厂大屏每日数据", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TElecdashboardDay tElecdashboardDay)
    {
        List<TElecdashboardDay> list = tElecdashboardDayService.selectTElecdashboardDayList(tElecdashboardDay);
        ExcelUtil<TElecdashboardDay> util = new ExcelUtil<TElecdashboardDay>(TElecdashboardDay.class);
        return util.exportExcel(list, "day");
    }

    /**
     * 获取电厂大屏每日数据详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tElecdashboardDayService.selectTElecdashboardDayById(id));
    }

    /**
     * 新增电厂大屏每日数据
     */
    @Log(title = "电厂大屏每日数据", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TElecdashboardDay tElecdashboardDay)
    {
        return toAjax(tElecdashboardDayService.insertTElecdashboardDay(tElecdashboardDay));
    }

    /**
     * 修改电厂大屏每日数据
     */
    @Log(title = "电厂大屏每日数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TElecdashboardDay tElecdashboardDay)
    {
        return toAjax(tElecdashboardDayService.updateTElecdashboardDay(tElecdashboardDay));
    }

    /**
     * 删除电厂大屏每日数据
     */
    @Log(title = "电厂大屏每日数据", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tElecdashboardDayService.deleteTElecdashboardDayByIds(ids));
    }
}
