package com.ruoyi.project.ehs.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.ehs.domain.TAutosprinkler;
import com.ruoyi.project.ehs.service.ITAutosprinklerService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 自动喷淋系统Controller
 *
 * @author ruoyi
 * @date 2020-11-25
 */
@RestController
@RequestMapping("/ehs/autosprinkler")
public class TAutosprinklerController extends BaseController
{
    @Autowired
    private ITAutosprinklerService tAutosprinklerService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    /**
     * 查询自动喷淋系统列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:list')")
    @GetMapping("/list")
    public TableDataInfo list(TAutosprinkler tAutosprinkler)
    {
        startPage();
        List<TAutosprinkler> list = tAutosprinklerService.selectTAutosprinklerList(tAutosprinkler);
        return getDataTable(list);
    }

    /**
     * 导出自动喷淋系统列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:export')")
    @Log(title = "自动喷淋系统", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TAutosprinkler tAutosprinkler)
    {
        List<TAutosprinkler> list = tAutosprinklerService.selectTAutosprinklerList(tAutosprinkler);
        ExcelUtil<TAutosprinkler> util = new ExcelUtil<TAutosprinkler>(TAutosprinkler.class);
        return util.exportExcel(list, "autosprinkler");
    }

    /**
     * 获取自动喷淋系统详细信息
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tAutosprinklerService.selectTAutosprinklerById(id));
    }

    /**
     * 新增自动喷淋系统
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:add')")
    @Log(title = "自动喷淋系统", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TAutosprinkler tAutosprinkler)
    {
        tAutosprinkler.setCreaterCode(getUserId().toString());
        return toAjax(tAutosprinklerService.insertTAutosprinkler(tAutosprinkler));
    }

    /**
     * 修改自动喷淋系统
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:edit')")
    @Log(title = "自动喷淋系统", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TAutosprinkler tAutosprinkler)
    {
        tAutosprinkler.setUpdaterCode(getUserId().toString());
        tAutosprinkler.setUpdatedate(new Date());
        return toAjax(tAutosprinklerService.updateTAutosprinkler(tAutosprinkler));
    }

    /**
     * 修改自动喷淋系统
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:edit')")
    @Log(title = "自动喷淋系统", businessType = BusinessType.OTHER)
    @RequestMapping("/batchEdit")
    public AjaxResult batchEdit(@RequestBody TAutosprinkler dto)
    {
        Long[] ids = dto.getIds();
        for (int i = 0; i < ids.length; i++) {
            TAutosprinkler t = new TAutosprinkler();
            t.setId(ids[i]);
            t.setRemarks(dto.getRemarks());
            t.setInspectstatus(dto.getInspectstatus());
            tAutosprinklerService.updateTAutosprinkler(t);
        }
        return AjaxResult.success();
    }

    /**
     * 删除自动喷淋系统
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:remove')")
    @Log(title = "自动喷淋系统", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tAutosprinklerService.deleteTAutosprinklerByIds(ids));
    }

    /**
     * 批量导入自动喷淋系统
     */
    @PreAuthorize("@ss.hasPermi('ehs:autosprinkler:add')")
    @Log(title = "自动喷淋系统", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TAutosprinkler> list = new ArrayList<TAutosprinkler>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        List<SysDictData> inspectstatus = iSysDictTypeService.selectDictDataByType("INSPECTSTATUS");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TAutosprinkler entity = new TAutosprinkler();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setLocation(cellValue);//部位
                    } else if (j == 2) {
                        entity.setSysname(cellValue);//系统名称
                    } else if (j == 3) {
                        entity.setChkcontents(cellValue);//检查内容
                    } else if (j == 4) {
                        entity.setManitenance(cellValue);//维修/检测情况
                    } else if (j == 5) {
                        entity.setInspector(cellValue);//检查人
                    } else if (j == 6) {
                        if (cellValue.length() > 3) {
                            entity.setInspectdate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//检查日期
                        }
                    } else if (j == 7) {
                        for (SysDictData p : inspectstatus) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setInspectstatus(p.getDictValue());//检查状况
                            }
                        }
                    } else if (j == 8) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 9) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TAutosprinkler t : list
        ) {
            failNum++;
            try {
                tAutosprinklerService.insertTAutosprinkler(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
