package com.ruoyi.project.ehs.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.ehs.domain.TEyewasher;
import com.ruoyi.project.ehs.service.ITEyewasherService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 洗眼器Controller
 *
 * @author ruoyi
 * @date 2020-11-24
 */
@RestController
@RequestMapping("/ehs/eyewasher")
public class TEyewasherController extends BaseController
{
    @Autowired
    private ITEyewasherService tEyewasherService;

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    /**
     * 查询洗眼器列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:list')")
    @GetMapping("/list")
    public TableDataInfo list(TEyewasher tEyewasher)
    {
        startPage();
        List<TEyewasher> list = tEyewasherService.selectTEyewasherList(tEyewasher);
        return getDataTable(list);
    }

    /**
     * 导出洗眼器列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:export')")
    @Log(title = "洗眼器", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TEyewasher tEyewasher)
    {
        List<TEyewasher> list = tEyewasherService.selectTEyewasherList(tEyewasher);
        ExcelUtil<TEyewasher> util = new ExcelUtil<TEyewasher>(TEyewasher.class);
        return util.exportExcel(list, "eyewasher");
    }

    /**
     * 获取洗眼器详细信息
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tEyewasherService.selectTEyewasherById(id));
    }

    /**
     * 新增洗眼器
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:add')")
    @Log(title = "洗眼器", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TEyewasher tEyewasher)
    {
        tEyewasher.setCreaterCode(getUserId().toString());
        return toAjax(tEyewasherService.insertTEyewasher(tEyewasher));
    }

    /**
     * 修改洗眼器
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:edit')")
    @Log(title = "洗眼器", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TEyewasher tEyewasher)
    {
        tEyewasher.setUpdaterCode(getUserId().toString());
        tEyewasher.setUpdatedate(new Date());
        return toAjax(tEyewasherService.updateTEyewasher(tEyewasher));
    }

    /**
     * 批量修改洗眼器
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:edit')")
    @Log(title = "洗眼器", businessType = BusinessType.OTHER)
    @RequestMapping("/batchEdit")
    public AjaxResult batchEdit(@RequestBody TEyewasher tEyewasher)
    {
        Long[] ids = tEyewasher.getIds();
        for (int i = 0; i < ids.length; i++) {
            TEyewasher t = new TEyewasher();
            t.setId(ids[i]);
            t.setRemarks(tEyewasher.getRemarks());
            t.setInspectstatus(tEyewasher.getInspectstatus());
            t.setUpdaterCode(getUserId().toString());
            t.setUpdatedate(new Date());
            tEyewasherService.updateTEyewasher(t);
        }
        return AjaxResult.success();
    }

    /**
     * 删除洗眼器
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:remove')")
    @Log(title = "洗眼器", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tEyewasherService.deleteTEyewasherByIds(ids));
    }

    /**
     * 批量导入洗眼器
     */
    @PreAuthorize("@ss.hasPermi('ehs:eyewasher:add')")
    @Log(title = "洗眼器", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TEyewasher> list = new ArrayList<TEyewasher>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        List<SysDictData> inspectstatus = iSysDictTypeService.selectDictDataByType("INSPECTSTATUS");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TEyewasher entity = new TEyewasher();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim().trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setName(cellValue);//名称
                    } else if (j == 2) {
                        entity.setArea(cellValue);//区域
                    } else if (j == 3) {
                        entity.setPosition(cellValue);//位置
                    } else if (j == 4) {
                        if (cellValue.length() > 3) {
                            entity.setInspectdate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));//检查日期
                        }
                    } else if (j == 5) {
                        for (SysDictData p : inspectstatus) {
                            if (p.getDictLabel().equals(cellValue.trim().trim())) {
                                entity.setInspectstatus(p.getDictValue());//检查状况
                            }
                        }
                    } else if (j == 6) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim().trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 7) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TEyewasher t : list
        ) {
            failNum++;
            try {
                tEyewasherService.insertTEyewasher(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
