package com.ruoyi.project.ehs.controller;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.alibaba.druid.sql.ast.TDDLHint;
import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysDictTypeService;
import org.apache.poi.ss.usermodel.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.ehs.domain.TWasteDanger;
import com.ruoyi.project.ehs.service.ITWasteDangerService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 危险废物清单Controller
 *
 * @author ruoyi
 * @date 2020-12-08
 */
@RestController
@RequestMapping("/ehs/danger")
public class TWasteDangerController extends BaseController
{
    @Autowired
    private ITWasteDangerService tWasteDangerService;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    @Autowired
    private ISysDeptService iSysDeptService;

    /**
     * 查询危险废物清单列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:danger:list')")
    @GetMapping("/list")
    public TableDataInfo list(TWasteDanger tWasteDanger)
    {
        startPage();
        List<TWasteDanger> list = tWasteDangerService.selectTWasteDangerList(tWasteDanger);
        return getDataTable(list);
    }

    /**
     * 导出危险废物清单列表
     */
    @PreAuthorize("@ss.hasPermi('ehs:danger:export')")
    @Log(title = "危险废物清单", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TWasteDanger tWasteDanger)
    {
        List<TWasteDanger> list = tWasteDangerService.selectTWasteDangerList(tWasteDanger);
        ExcelUtil<TWasteDanger> util = new ExcelUtil<TWasteDanger>(TWasteDanger.class);
        return util.exportExcel(list, "danger");
    }

    /**
     * 获取危险废物清单详细信息
     */
    @PreAuthorize("@ss.hasPermi('ehs:danger:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tWasteDangerService.selectTWasteDangerById(id));
    }

    /**
     * 新增危险废物清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:danger:add')")
    @Log(title = "危险废物清单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TWasteDanger tWasteDanger)
    {
        tWasteDanger.setCreaterCode(getUserId().toString());
        return toAjax(tWasteDangerService.insertTWasteDanger(tWasteDanger));
    }

    /**
     * 修改危险废物清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:danger:edit')")
    @Log(title = "危险废物清单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TWasteDanger tWasteDanger)
    {
        tWasteDanger.setUpdaterCode(getUserId().toString());
        tWasteDanger.setUpdatedate(new Date());
        return toAjax(tWasteDangerService.updateTWasteDanger(tWasteDanger));
    }

    /**
     * 删除危险废物清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:danger:remove')")
    @Log(title = "危险废物清单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tWasteDangerService.deleteTWasteDangerByIds(ids));
    }

    /**
     * 批量导入危险废物清单
     */
    @PreAuthorize("@ss.hasPermi('ehs:danger:add')")
    @Log(title = "危险废物清单", businessType = BusinessType.INSERT)
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TWasteDanger> list = new ArrayList<TWasteDanger>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //字典查询
        List<SysDictData> plant = iSysDictTypeService.selectDictDataByType("PLANT_DIVIDE");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 1; i < rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TWasteDanger entity = new TWasteDanger();
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    //  cell.setCellType(CellType.STRING);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        for (SysDictData p : plant) {
                            if (p.getDictLabel().equals(cellValue.trim())) {
                                entity.setPlantCode(p.getDictValue());//装置名称
                            }
                        }
                    } else if (j == 1) {
                        entity.setName(cellValue);//危废名称
                    } else if (j == 2) {
                        entity.setCategory(cellValue);//危废类别
                    } else if (j == 3) {
                        entity.setCode(cellValue);//危废代码
                    } else if (j == 4) {
                        entity.setDescription(cellValue);//危险废物描述
                    } else if (j == 5) {
                        entity.setDisposalMode(cellValue);//处置利用方式
                    } else if (j == 6) {
                        entity.setProductLink(cellValue);//生产环节
                    } else if (j == 7) {
                        entity.setStoragePlace(cellValue);//贮存场所
                    } else if (j == 8) {
                        entity.setDisposalDestination(cellValue);//处置方式及去向
                    } else if (j == 9) {
                        entity.setOwner(cellValue);//责任人
                    } else if (j == 10) {
                        entity.setHazardsSpec(cellValue);//危险特性
                    } else if (j == 11) {
                        for (SysDept d : dept) {
                            if (d.getDeptName().equals(cellValue.trim())) {
                                entity.setDeptId(d.getDeptId());//部门编号
                            }
                        }
                    } else if (j == 12) {
                        entity.setRemarks(cellValue);//备注
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TWasteDanger t : list
        ) {
            failNum++;
            try {
                tWasteDangerService.insertTWasteDanger(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success(String.valueOf(successNumber), failRow);
    }
}
