package com.ruoyi.project.monitor.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.monitor.mapper.TPfdMapper;
import com.ruoyi.project.monitor.domain.TPfd;
import com.ruoyi.project.monitor.service.ITPfdService;

/**
 * PFD管理Service业务层处理
 *
 * @author ssy
 * @date 2024-06-24
 */
@Service
public class TPfdServiceImpl implements ITPfdService
{
    @Autowired
    private TPfdMapper tPfdMapper;

    /**
     * 查询PFD管理
     *
     * @param id PFD管理ID
     * @return PFD管理
     */
    @Override
    public TPfd selectTPfdById(Long id)
    {
        return tPfdMapper.selectTPfdById(id);
    }

    /**
     * 查询PFD管理
     *
     * @param code PFD管理编号
     * @return PFD管理
     */
    public TPfd selectTPfdByCode(String code) {
        return tPfdMapper.selectTPfdByCode(code);
    }

    /**
     * 查询PFD管理列表
     *
     * @param tPfd PFD管理
     * @return PFD管理
     */
    @Override
    public List<TPfd> selectTPfdList(TPfd tPfd)
    {
        return tPfdMapper.selectTPfdList(tPfd);
    }

    /**
     * 新增PFD管理
     *
     * @param tPfd PFD管理
     * @return 结果
     */
    @Override
    public int insertTPfd(TPfd tPfd)
    {
        tPfd.setCreateTime(DateUtils.getNowDate());
        return tPfdMapper.insertTPfd(tPfd);
    }

    /**
     * 修改PFD管理
     *
     * @param tPfd PFD管理
     * @return 结果
     */
    @Override
    public int updateTPfd(TPfd tPfd)
    {
        tPfd.setUpdateTime(DateUtils.getNowDate());
        return tPfdMapper.updateTPfd(tPfd);
    }

    /**
     * 批量删除PFD管理
     *
     * @param ids 需要删除的PFD管理ID
     * @return 结果
     */
    @Override
    public int deleteTPfdByIds(Long[] ids)
    {
        return tPfdMapper.deleteTPfdByIds(ids);
    }

    /**
     * 删除PFD管理信息
     *
     * @param id PFD管理ID
     * @return 结果
     */
    @Override
    public int deleteTPfdById(Long id)
    {
        return tPfdMapper.deleteTPfdById(id);
    }
}
