package com.ruoyi.project.patrol.fire.controller;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.interceptor.annotation.RepeatSubmit;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.patrol.fire.domain.TFireSandbox;
import com.ruoyi.project.patrol.fire.service.ITFireSandboxService;
import com.ruoyi.project.patrol.patrol.domain.TPatrolIssues;
import com.ruoyi.project.patrol.patrol.domain.TPatrolPlan;
import com.ruoyi.project.patrol.patrol.service.ITPatrolIssuesService;
import com.ruoyi.project.patrol.patrol.service.ITPatrolPlanService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

/**
 * 应急沙箱Controller
 *
 * @author admin
 * @date 2023-11-02
 */
@RestController
@RequestMapping("/fire/sandbox")
public class TFireSandboxController extends BaseController {
    @Autowired
    private ITFireSandboxService tFireSandboxService;

    @Autowired
    private ITPatrolPlanService tPatrolPlanService;

    @Autowired
    private ITPatrolIssuesService tPatrolIssuesService;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 查询应急沙箱列表
     */
    @PreAuthorize("@ss.hasPermi('fire:sandbox:list')")
    @GetMapping("/list")
    public TableDataInfo list(TFireSandbox tFireSandbox) {
        startPage();
        if (StringUtils.isEmpty(tFireSandbox.getIssuesFlag()))
            tFireSandbox.setIssuesFlag("0");
        List<TFireSandbox> list = tFireSandboxService.selectTFireSandboxList(tFireSandbox);
        return getDataTable(list);
    }

    /**
     * 导出应急沙箱列表
     */
    @PreAuthorize("@ss.hasPermi('fire:sandbox:export')")
    @Log(title = "应急沙箱", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public AjaxResult export(TFireSandbox tFireSandbox) {
        try {
            tFireSandbox.setIssuesFlag("0");
            List<TFireSandbox> list = tFireSandboxService.selectTFireSandboxList(tFireSandbox);
            XSSFWorkbook workbook = new XSSFWorkbook();
            // 创建工作簿
            XSSFSheet sheet = workbook.createSheet();
            // 设置列宽
            sheet.setColumnWidth(0, 1200);
            sheet.setColumnWidth(1, 3500);
            sheet.setColumnWidth(2, 3500);
            sheet.setColumnWidth(3, 4500);
            sheet.setColumnWidth(4, 4500);
            sheet.setColumnWidth(5, 4500);
            sheet.setColumnWidth(6, 4500);
            //第一行
            XSSFRow row1 = sheet.createRow(0);
            row1.createCell(0).setCellValue("CBP-6.3.0-SCFSH-020.15  Rev:04  2021/10/01");
            ExcelUtils.createVoidCell(1, 6, row1);
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 6));//合并
            //第二行
            XSSFRow row2 = sheet.createRow(1);
            row2.createCell(0).setCellValue("应急沙箱检查记录");
            ExcelUtils.createVoidCell(1, 6, row2);
            sheet.addMergedRegion(new CellRangeAddress(1, 1, 0, 6));
            //第三行
            XSSFRow row3 = sheet.createRow(2);
            row3.createCell(0).setCellValue("年度：" + tFireSandbox.getYear());
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 2));
            ExcelUtils.createVoidCell(1, 6, row3);
            row3.createCell(3).setCellValue("检查频率：1次/月");
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 3, 6));
            //第四行
            XSSFRow row4 = sheet.createRow(3);
            row4.createCell(0).setCellValue("序号");
            row4.createCell(1).setCellValue("位号");
            row4.createCell(2).setCellValue("位置");
            row4.createCell(3).setCellValue("检查内容");
            ExcelUtils.createVoidCell(4, 6, row4);
            sheet.addMergedRegion(new CellRangeAddress(3, 3, 3, 6));
            //第五行
            XSSFRow row5 = sheet.createRow(4);
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 0, 0));
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 1, 1));
            sheet.addMergedRegion(new CellRangeAddress(3, 4, 2, 2));
            ExcelUtils.createVoidCell(0, 2, row5);
            row5.createCell(3).setCellValue("沙箱外观");
            row5.createCell(4).setCellValue("标识完好度");
            row5.createCell(5).setCellValue("吸油棉数量");
            row5.createCell(6).setCellValue("检查人");
            Long planId = null;
            //遍历数组赋值
            for (int i = 0; i < list.size(); i++) {
                TFireSandbox paga = list.get(i);
                planId = paga.getPlanId();
                XSSFRow row = sheet.createRow(5 + i);
                row.createCell(0).setCellValue(i + 1);
                row.createCell(1).setCellValue(paga.getDevNo());
                row.createCell(2).setCellValue(paga.getPosition());
                row.createCell(3).setCellValue("1".equals(paga.getSandboxAppearance()) ? "√" : paga.getSandboxAppearance());
                row.createCell(4).setCellValue("1".equals(paga.getSignIntegrity()) ? "√" : paga.getSignIntegrity());
                row.createCell(5).setCellValue("1".equals(paga.getCottonQuantity()) ? "√" : paga.getCottonQuantity());
                XSSFCell cell10 = row.createCell(6);
                if (StringUtils.isNotEmpty(paga.getUpdaterCode())) {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(paga.getUpdaterCode()));
                    cell10.setCellValue(sysUser == null ? "" : sysUser.getNickName());
                }
            }
            String confirmed = "";
            Date confirmedDate = null;
            TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(planId);
            if (Objects.nonNull(tPatrolPlan) && Objects.nonNull(tPatrolPlan.getConfirmed())) {
                SysUser sysUser = sysUserService.selectUserById(Long.valueOf(tPatrolPlan.getConfirmed()));            if (sysUser!=null) {
                confirmed = sysUser.getSignUrl();
                confirmedDate = tPatrolPlan.getConfirmDate();
            }
            }
            // 循环后下一行
            int nextRowNum = 5 + list.size();
            XSSFRow row28 = sheet.createRow(nextRowNum);
            row28.setHeightInPoints(30);
            XSSFCell row28Cell1 = row28.createCell(0);
            row28Cell1.setCellValue("签名");
            ExcelUtils.createVoidCell(1, 2, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 2));
            if (StringUtils.isNotEmpty(confirmed))
                ExcelUtils.insertPicture(workbook, sheet, confirmed, nextRowNum, 5, 1, 1);
            ExcelUtils.createVoidCell(3, 6, row28);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 3, 6));
            // 下一行
            nextRowNum++;
            XSSFRow row29 = sheet.createRow(nextRowNum);
            row29.setHeightInPoints(30);
            XSSFCell row29Cell1 = row29.createCell(0);
            row29Cell1.setCellValue("日期");
            ExcelUtils.createVoidCell(1, 2, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 2));
            row29.createCell(3).setCellValue(confirmedDate != null ? new SimpleDateFormat("yyyy-MM-dd").format(confirmedDate) : "");
            ExcelUtils.createVoidCell(4, 6, row29);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 3, 6));
            // 设置单元格样式
            // 文字居中，自动换行，四周边框
            CellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);
            style.setWrapText(true);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);

            // 设置垂直居中，四周边框
            CellStyle style2 = workbook.createCellStyle();
            style2.setVerticalAlignment(VerticalAlignment.CENTER);
            style2.setBorderTop(BorderStyle.THIN);
            style2.setBorderBottom(BorderStyle.THIN);
            style2.setBorderLeft(BorderStyle.THIN);
            style2.setBorderRight(BorderStyle.THIN);
            for (Row row : sheet) {
                row.setHeightInPoints(30);
                if (row.getRowNum() == 0) {
                    continue;
                }
                if (row.getRowNum() == 2) {
                    for (Cell cell : row) {
                        if (cell.getColumnIndex() == 0 || cell.getColumnIndex() == 1 || cell.getColumnIndex() == 2)
                            cell.setCellStyle(style2);
                        else
                            cell.setCellStyle(style);
                    }
                } else {
                    for (Cell cell : row) {
                        cell.setCellStyle(style);
                    }
                }
            }
            nextRowNum++;
            XSSFRow row30 = sheet.createRow(nextRowNum);
            row30.createCell(0).setCellValue("说明：");
            ExcelUtils.createVoidCell(1, 6, row30);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            nextRowNum++;
            XSSFRow row31 = sheet.createRow(nextRowNum);
            row31.createCell(0).setCellValue("1.检查正常打“√”。");
            ExcelUtils.createVoidCell(1, 6, row31);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            nextRowNum++;
            XSSFRow row32 = sheet.createRow(nextRowNum);
            row32.createCell(0).setCellValue("2.检查有不正常情况打“X”，并记录存在问题，处理结果及完成时间。");
            ExcelUtils.createVoidCell(1, 6, row32);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            nextRowNum++;
            XSSFRow row33 = sheet.createRow(nextRowNum);
            row33.createCell(0).setCellValue("3. 检查频率为每1个月检查维护一次。");
            ExcelUtils.createVoidCell(1, 6, row33);
            sheet.addMergedRegion(new CellRangeAddress(nextRowNum, nextRowNum, 0, 6));

            //返回
            String fileName =  "PSSR_13_消气防设施_应急沙箱检查记录_" + UUID.randomUUID().toString() + ".xlsx";
            OutputStream os = Files.newOutputStream(Paths.get(ExcelUtil.getAbsoluteFile(fileName)));
            workbook.write(os);
            workbook.close();
            return AjaxResult.success(fileName);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取应急沙箱详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:sandbox:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tFireSandboxService.selectTFireSandboxById(id));
    }

    /**
     * 获取应急沙箱详细信息
     */
    @PreAuthorize("@ss.hasPermi('fire:sandbox:query')")
    @GetMapping(value = "/info")
    public AjaxResult getInfo(TFireSandbox tFireSandbox) {
        if (StringUtils.isEmpty(tFireSandbox.getDevNo()) || StringUtils.isNull(tFireSandbox.getPlanId())) {
            return AjaxResult.error("位号或计划id不能为空！");
        }
        TFireSandbox item = new TFireSandbox();
        item.setPlanId(tFireSandbox.getPlanId());
        item.setPrecision(tFireSandbox.getDevNo());
        item.setIssuesFlag("0");
        List<TFireSandbox> items = tFireSandboxService.selectTFireSandboxList(item);
        if (CollectionUtils.isNotEmpty(items))
            return AjaxResult.success(items.get(0));
        return AjaxResult.error("此设备不在当前巡检计划中!");
    }

    /**
     * 新增应急沙箱
     */
    @PreAuthorize("@ss.hasPermi('fire:sandbox:add')")
    @Log(title = "应急沙箱", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TFireSandbox tFireSandbox) {
        return toAjax(tFireSandboxService.insertTFireSandbox(tFireSandbox));
    }

    /**
     * 修改应急沙箱
     */
    @PreAuthorize("@ss.hasPermi('fire:sandbox:edit')")
    @Log(title = "应急沙箱", businessType = BusinessType.UPDATE)
    @PutMapping
    @RepeatSubmit
    public AjaxResult edit(@RequestBody TFireSandbox tFireSandbox) {
        tFireSandbox.setUpdatedate(new Date());
        tFireSandbox.setUpdaterCode(getUserId().toString());
        tFireSandbox.setCheckStatus("1");
        tFireSandbox.setCheckDate(new Date());
        TFireSandbox item = tFireSandboxService.selectTFireSandboxById(tFireSandbox.getId());
        TPatrolPlan tPatrolPlan = tPatrolPlanService.selectTPatrolPlanById(tFireSandbox.getPlanId());
        if (tPatrolPlan.getStatus()==1){
            return AjaxResult.error("当前选择的计划已确认！");
        }
        if (!"1".equals(item.getCheckStatus())) {
            tPatrolPlan.setUncheckedNum(tPatrolPlan.getUncheckedNum() - 1);
            tPatrolPlan.setCheckedNum(tPatrolPlan.getCheckedNum() + 1);
            tPatrolPlanService.updateTPatrolPlan(tPatrolPlan);
        }
        if ((StringUtils.isNotEmpty(tFireSandbox.getCottonQuantity()) && !"1".equals(tFireSandbox.getCottonQuantity())) ||
                (StringUtils.isNotEmpty(tFireSandbox.getSandboxAppearance()) && !"1".equals(tFireSandbox.getSandboxAppearance())) ||
                (StringUtils.isNotEmpty(tFireSandbox.getSignIntegrity()) && !"1".equals(tFireSandbox.getSignIntegrity()))) {
            tFireSandbox.setIssuesStatus("1");
            tFireSandbox.setCheckStatus("0");
            tFireSandboxService.updateTFireSandbox(tFireSandbox);
            tFireSandbox.setIssuesFlag("1");
            tFireSandbox.setCheckStatus("1");
            tFireSandbox.setCreatedate(new Date());
            tFireSandbox.setCreaterCode(getUserId().toString());
            tFireSandbox.setDevNo(item.getDevNo());
            tFireSandbox.setPosition(item.getPosition());
            tFireSandboxService.insertTFireSandbox(tFireSandbox);
            TPatrolIssues tPatrolIssues = new TPatrolIssues();
            tPatrolIssues.setPatrolType("13");
            tPatrolIssues.setDevDescribe(tFireSandbox.getDevNo() + " " + tFireSandbox.getPosition());
            tPatrolIssues.setLinkId(tFireSandbox.getId());
            tPatrolIssues.setStatus(0L);
            tPatrolIssuesService.insertTPatrolIssues(tPatrolIssues);
        } else {
            if ("1".equals(item.getIssuesStatus())) {
                TPatrolIssues tPatrolIssues = new TPatrolIssues();
                tPatrolIssues.setPatrolType("13");
                tPatrolIssues.setDevDescribe(item.getDevNo() + " " + item.getPosition());
                tPatrolIssues.setCreatedate(new Date());
                for (TPatrolIssues patrolIssues : tPatrolIssuesService.selectTPatrolIssuesList(tPatrolIssues)) {
                    patrolIssues.setStatus(1L);
                    tPatrolIssuesService.updateTPatrolIssues(patrolIssues);
                }
            }
            tFireSandbox.setIssuesStatus("0");
            tFireSandboxService.updateTFireSandbox(tFireSandbox);
        }
        return AjaxResult.success();
    }

    /**
     * 删除应急沙箱
     */
    @PreAuthorize("@ss.hasPermi('fire:sandbox:remove')")
    @Log(title = "应急沙箱", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tFireSandboxService.deleteTFireSandboxByIds(ids));
    }
}
