package com.ruoyi.project.production.controller;

import java.util.Date;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TMonthlyQualityReportS3010;
import com.ruoyi.project.production.service.ITMonthlyQualityReportS3010Service;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 质量月报S3010Controller
 *
 * @author ruoyi
 * @date 2023-10-27
 */
@RestController
@RequestMapping("/production/s3010")
public class TMonthlyQualityReportS3010Controller extends BaseController
{
    @Autowired
    private ITMonthlyQualityReportS3010Service tMonthlyQualityReportS3010Service;

    /**
     * 查询质量月报S3010列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:list')")
    @GetMapping("/list")
    public AjaxResult list(TMonthlyQualityReportS3010 tMonthlyQualityReportS3010)
    {
        Date endDate = tMonthlyQualityReportS3010.getEndDate();
        endDate.setHours(23);
        endDate.setMinutes(59);
        endDate.setSeconds(59);
        tMonthlyQualityReportS3010.setEndDate(endDate);
        return AjaxResult.success(tMonthlyQualityReportS3010Service.selectTMonthlyQualityReportS3010List(tMonthlyQualityReportS3010));
    }

    /**
     * 导出质量月报S3010列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:export')")
    @Log(title = "质量月报S3010", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TMonthlyQualityReportS3010 tMonthlyQualityReportS3010)
    {
        List<TMonthlyQualityReportS3010> list = tMonthlyQualityReportS3010Service.selectTMonthlyQualityReportS3010List(tMonthlyQualityReportS3010);
        ExcelUtil<TMonthlyQualityReportS3010> util = new ExcelUtil<TMonthlyQualityReportS3010>(TMonthlyQualityReportS3010.class);
        return util.exportExcel(list, "quality");
    }

    /**
     * 获取质量月报S3010详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:quality:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tMonthlyQualityReportS3010Service.selectTMonthlyQualityReportS3010ById(id));
    }

    /**
     * 新增质量月报S3010
     */
    @PreAuthorize("@ss.hasPermi('production:quality:add')")
    @Log(title = "质量月报S3010", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TMonthlyQualityReportS3010 tMonthlyQualityReportS3010)
    {
        return toAjax(tMonthlyQualityReportS3010Service.insertTMonthlyQualityReportS3010(tMonthlyQualityReportS3010));
    }

    /**
     * 修改质量月报S3010
     */
    @PreAuthorize("@ss.hasPermi('production:quality:edit')")
    @Log(title = "质量月报S3010", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TMonthlyQualityReportS3010 tMonthlyQualityReportS3010)
    {
        return toAjax(tMonthlyQualityReportS3010Service.updateTMonthlyQualityReportS3010(tMonthlyQualityReportS3010));
    }

    /**
     * 删除质量月报S3010
     */
    @PreAuthorize("@ss.hasPermi('production:quality:remove')")
    @Log(title = "质量月报S3010", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tMonthlyQualityReportS3010Service.deleteTMonthlyQualityReportS3010ByIds(ids));
    }
}
