package com.ruoyi.project.production.controller;

import java.util.Date;
import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TMonthlyQualityReportS3011;
import com.ruoyi.project.production.service.ITMonthlyQualityReportS3011Service;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 质量月报S3011Controller
 *
 * @author ruoyi
 * @date 2023-10-27
 */
@RestController
@RequestMapping("/production/s3011")
public class TMonthlyQualityReportS3011Controller extends BaseController
{
    @Autowired
    private ITMonthlyQualityReportS3011Service tMonthlyQualityReportS3011Service;

    /**
     * 查询质量月报S3011列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:list')")
    @GetMapping("/list")
    public AjaxResult list(TMonthlyQualityReportS3011 tMonthlyQualityReportS3011)
    {
        Date endDate = tMonthlyQualityReportS3011.getEndDate();
        endDate.setHours(23);
        endDate.setMinutes(59);
        endDate.setSeconds(59);
        tMonthlyQualityReportS3011.setEndDate(endDate);
        return AjaxResult.success(tMonthlyQualityReportS3011Service.selectTMonthlyQualityReportS3011List(tMonthlyQualityReportS3011));
    }

    /**
     * 导出质量月报S3011列表
     */
    @PreAuthorize("@ss.hasPermi('production:quality:export')")
    @Log(title = "质量月报S3011", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TMonthlyQualityReportS3011 tMonthlyQualityReportS3011)
    {
        List<TMonthlyQualityReportS3011> list = tMonthlyQualityReportS3011Service.selectTMonthlyQualityReportS3011List(tMonthlyQualityReportS3011);
        ExcelUtil<TMonthlyQualityReportS3011> util = new ExcelUtil<TMonthlyQualityReportS3011>(TMonthlyQualityReportS3011.class);
        return util.exportExcel(list, "quality");
    }

    /**
     * 获取质量月报S3011详细信息
     */
    @PreAuthorize("@ss.hasPermi('production:quality:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tMonthlyQualityReportS3011Service.selectTMonthlyQualityReportS3011ById(id));
    }

    /**
     * 新增质量月报S3011
     */
    @PreAuthorize("@ss.hasPermi('production:quality:add')")
    @Log(title = "质量月报S3011", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TMonthlyQualityReportS3011 tMonthlyQualityReportS3011)
    {
        return toAjax(tMonthlyQualityReportS3011Service.insertTMonthlyQualityReportS3011(tMonthlyQualityReportS3011));
    }

    /**
     * 修改质量月报S3011
     */
    @PreAuthorize("@ss.hasPermi('production:quality:edit')")
    @Log(title = "质量月报S3011", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TMonthlyQualityReportS3011 tMonthlyQualityReportS3011)
    {
        return toAjax(tMonthlyQualityReportS3011Service.updateTMonthlyQualityReportS3011(tMonthlyQualityReportS3011));
    }

    /**
     * 删除质量月报S3011
     */
    @PreAuthorize("@ss.hasPermi('production:quality:remove')")
    @Log(title = "质量月报S3011", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tMonthlyQualityReportS3011Service.deleteTMonthlyQualityReportS3011ByIds(ids));
    }
}
