package com.ruoyi.project.production.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.production.domain.TPpeWardrobe;
import com.ruoyi.project.production.service.ITPpeWardrobeService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 更衣柜Controller
 *
 * @author ssy
 * @date 2023-11-30
 */
@RestController
@RequestMapping("/production/wardrobe")
public class TPpeWardrobeController extends BaseController
{
    @Autowired
    private ITPpeWardrobeService tPpeWardrobeService;

    /**
     * 查询更衣柜列表
     */

    @GetMapping("/list")
    public TableDataInfo list(TPpeWardrobe tPpeWardrobe)
    {
        startPage();
        List<TPpeWardrobe> list = tPpeWardrobeService.selectTPpeWardrobeList(tPpeWardrobe);
        return getDataTable(list);
    }

    /**
     * 导出更衣柜列表
     */

    @Log(title = "更衣柜", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPpeWardrobe tPpeWardrobe)
    {
        List<TPpeWardrobe> list = tPpeWardrobeService.selectTPpeWardrobeList(tPpeWardrobe);
        ExcelUtil<TPpeWardrobe> util = new ExcelUtil<TPpeWardrobe>(TPpeWardrobe.class);
        return util.exportExcel(list, "wardrobe");
    }

    /**
     * 获取更衣柜详细信息
     */

    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tPpeWardrobeService.selectTPpeWardrobeById(id));
    }

    /**
     * 新增更衣柜
     */

    @Log(title = "更衣柜", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPpeWardrobe tPpeWardrobe)
    {
        return toAjax(tPpeWardrobeService.insertTPpeWardrobe(tPpeWardrobe));
    }

    /**
     * 修改更衣柜
     */

    @Log(title = "更衣柜", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPpeWardrobe tPpeWardrobe)
    {
        return toAjax(tPpeWardrobeService.updateTPpeWardrobe(tPpeWardrobe));
    }

    /**
     * 删除更衣柜
     */

    @Log(title = "更衣柜", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tPpeWardrobeService.deleteTPpeWardrobeByIds(ids));
    }
}
