package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.pssr.domain.*;
import com.ruoyi.project.pssr.mapper.TPssrAnalyzerMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.ProcessEngine;
import org.activiti.engine.ProcessEngines;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 在线分析仪Controller
 *
 * @author ssy
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/pssr/analyzer")
public class TPssrAnalyzerController extends BaseController {
    @Autowired
    private ITPssrFileService tPssrFileService;

    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrAnalyzerService tPssrAnalyzerService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private ISysUserService sysUserService;
    @Resource
    private TPssrAnalyzerMapper tPssrAnalyzerMapper;

    private String forShort = "zxfxy";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;

    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrAnalyzer> list = new ArrayList<TPssrAnalyzer>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrAnalyzer entity = new TPssrAnalyzer();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setUnit(cellValue);
                    } else if (j == 1) {
                        entity.setDevNo(cellValue);
                    } else if (j == 2) {
                        entity.setVisualInspection(cellValue);
                    } else if (j == 3) {
                        entity.setFunctionalCheck(cellValue);
                    } else if (j == 4) {
                        if (cellValue.length() > 3) {
                            entity.setConfirmationDate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));
                        }
                    } else if (j == 5) {
                        entity.setRemarks(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrAnalyzer t : list
        ) {
            failNum++;
            try {
                this.add(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }

    /**
     * 查询在线分析仪列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrAnalyzer tPssrAnalyzer) {
        startPage();
        List<TPssrAnalyzer> list = tPssrAnalyzerService.selectTPssrAnalyzerList(tPssrAnalyzer);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), forShort));
        });
        return getDataTable(list);
    }

    /**
     * 导出在线分析仪列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:export')")
    @Log(title = "在线分析仪", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrAnalyzer tPssrAnalyzer) {
        List<TPssrAnalyzer> list = tPssrAnalyzerService.selectTPssrAnalyzerList(tPssrAnalyzer);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrAnalyzer> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/zxfxy.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 3;
            int num = 1;

            Row originalRow = sheet.getRow(3);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrAnalyzer t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getUnit());
                row.createCell(2).setCellValue(t.getDevNo());
                row.createCell(3).setCellValue(t.getVisualInspection());
                row.createCell(4).setCellValue(t.getFunctionalCheck());
                row.createCell(5);
                row.createCell(6);
                try {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(t.getConfirm1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirm2()));
                    String confirm1 = sysUser.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 5, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 6, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(7).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(8).setCellValue(t.getRemarks());


                //渲染样式
                for (int i = 0; i < 9; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename =  "PSSR_31_在线分析仪_" + UUID.randomUUID().toString() + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取在线分析仪详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrAnalyzer item = tPssrAnalyzerService.selectTPssrAnalyzerById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), forShort));
        return AjaxResult.success(item);
    }

    /**
     * 新增在线分析仪
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:add')")
    @Log(title = "在线分析仪", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrAnalyzer tPssrAnalyzer) {
        tPssrAnalyzer.setApproveStatus(0L);
        return toAjax(tPssrAnalyzerService.insertTPssrAnalyzer(tPssrAnalyzer));
    }

    /**
     * 修改在线分析仪
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:edit')")
    @Log(title = "在线分析仪", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrAnalyzer tPssrAnalyzer) {
        tPssrFileService.updateFileRelevance(tPssrAnalyzer.getFileIds(), forShort, tPssrAnalyzer.getId(), tPssrAnalyzer.getSubId());
        return toAjax(tPssrAnalyzerService.updateTPssrAnalyzer(tPssrAnalyzer));
    }

    /**
     * 修改在线分析仪
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:edit')")
    @Log(title = "在线分析仪", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrAnalyzer tPssrAnalyzer) {
        return toAjax(tPssrAnalyzerMapper.updateTPssrAnalyzerByIds(tPssrAnalyzer));
    }

    /**
     * 删除在线分析仪
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:remove')")
    @Log(title = "在线分析仪", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrAnalyzerService.deleteTPssrAnalyzerByIds(ids));
    }


    /**
     * 确认在线分析仪
     */
    @PreAuthorize("@ss.hasPermi('pssr:analyzer:edit')")
    @Log(title = "在线分析仪", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmAnalyzer")
    public AjaxResult Analyzer(@RequestBody TPssrAnalyzer tPssrAnalyzer) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        if (tPssrAnalyzer.getTaskType() == 4) {
            //拆锁确认
            queryStatus = 1;
            approveStatus = 3;
            date = new Date();
        } else if (tPssrAnalyzer.getTaskType() == 5) {
            //上锁确认
            queryStatus = 3;
            approveStatus = 2;
            date = new Date();
        }
        // 修改状态
        if (tPssrAnalyzer.getIds() != null && tPssrAnalyzer.getIds().length > 0) {
            for (Long id : tPssrAnalyzer.getIds()) {
                TPssrAnalyzer item = tPssrAnalyzerService.selectTPssrAnalyzerById(id);
                item.setApproveStatus(approveStatus);
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                tPssrAnalyzerService.updateTPssrAnalyzer(item);
            }
        } else {
            TPssrAnalyzer lock = new TPssrAnalyzer();
            lock.setSubId(tPssrAnalyzer.getSubId());
            lock.setApproveStatus(queryStatus);
            for (TPssrAnalyzer item : tPssrAnalyzerService.selectTPssrAnalyzerList(lock)) {
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                item.setApproveStatus(approveStatus);
                tPssrAnalyzerService.updateTPssrAnalyzer(item);
            }
        }
        //查询当前待审批的确认人
        TPssrAnalyzer entity = new TPssrAnalyzer();
        entity.setSubId(tPssrAnalyzer.getSubId());
        entity.setApproveStatus(queryStatus);
        for (TPssrAnalyzer item : tPssrAnalyzerService.selectTPssrAnalyzerList(entity)) {
            if (tPssrAnalyzer.getTaskType() == 4) {
                if (item.getConfirm1().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            } else if (tPssrAnalyzer.getTaskType() == 5) {
                if (item.getConfirm2().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }

        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrAnalyzer.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();

    }

    /**
     * 驳回在线分析仪
     */
    @PutMapping("/turnDownAnalyzer")
    public AjaxResult turnDownAnalyzer(@RequestBody List<TPssrAnalyzer> tPssrAnalyzer) {
        if (CollectionUtils.isNotEmpty(tPssrAnalyzer)){
            String userId = getUserId().toString();
            Long subId = tPssrAnalyzer.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrAnalyzer analyzer : tPssrAnalyzer) {
                TPssrAnalyzer blind = new TPssrAnalyzer();
                blind.setId(analyzer.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrAnalyzerService.updateTPssrAnalyzer(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(analyzer.getSubId());
                turndown.setItemId(analyzer.getId());
                turndown.setReason(analyzer.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            ProcessEngine processEngine = ProcessEngines.getDefaultProcessEngine();
            TaskService taskService = processEngine.getTaskService();
            Task task = processEngine.getTaskService()//获取任务service
                    .createTaskQuery()//创建查询对象
                    .taskAssignee(userId)
                    .processInstanceId(approve.getProcessId()).singleResult();
            String taskId = task.getId();

            // 驳回 查询所有待审批的人员
            TPssrAnalyzer blind = new TPssrAnalyzer();
            blind.setSubId(subId);
            blind.setApproveStatus(1L);
            Set<String> installer = new HashSet<>();
            Set<String> remover = new HashSet<>();
            for (TPssrAnalyzer item : tPssrAnalyzerService.selectTPssrAnalyzerList(blind)) {

                // 安装人员
                installer.add(item.getConfirm1());

                //拆除人员
                remover.add(item.getConfirm2());
            }


            //处理流程节点
            Map<String, Object> param = new HashMap<>();
            param.put("condition", 1);
            param.put("confirmUsers1", new ArrayList<>(installer));
            param.put("confirmUsers2", new ArrayList<>(remover));
            //认领任务
            processEngine.getTaskService().claim(taskId, userId);
            taskService.addComment(taskId, approve.getProcessId(), "驳回");
            taskService.complete(taskId, param);

            // 修改审批表和sub表
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
