package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.pssr.domain.*;
import com.ruoyi.project.pssr.mapper.TPssrFrameMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.ProcessEngine;
import org.activiti.engine.ProcessEngines;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 支（吊）架Controller
 *
 * @author ssy
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/pssr/frame")
public class TPssrFrameController extends BaseController {
    @Autowired
    private ITPssrFileService tPssrFileService;
    @Resource
    private TPssrFrameMapper tPssrFrameMapper;
    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrFrameService tPssrFrameService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "zdj";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;

    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrFrame> list = new ArrayList<TPssrFrame>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrFrame entity = new TPssrFrame();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setBracketNumber(cellValue);
                    } else if (j == 1) {
                        entity.setPipelineNumber(cellValue);
                    } else if (j == 2) {
                        entity.setDevNo(cellValue);
                    } else if (j == 3) {
                        entity.setSupportType(cellValue);
                    } else if (j == 4) {
                        entity.setFramePipe(cellValue);
                    } else if (j == 5) {
                        entity.setFrameRoot(cellValue);
                    } else if (j == 6) {
                        entity.setFunctionalPart(cellValue);
                    } else if (j == 7) {
                        entity.setConnector(cellValue);
                    } else if (j == 8) {
                        if (cellValue.length() > 3) {
                            entity.setConfirmationDate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));
                        }
                    } else if (j == 9) {
                        entity.setRemarks(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrFrame t : list
        ) {
            failNum++;
            try {
                this.add(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }

    /**
     * 查询支（吊）架列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrFrame tPssrFrame) {
        startPage();
        List<TPssrFrame> list = tPssrFrameService.selectTPssrFrameList(tPssrFrame);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), forShort));
        });
        return getDataTable(list);
    }

    /**
     * 导出支（吊）架列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:export')")
    @Log(title = "支（吊）架", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrFrame tPssrFrame) {
        List<TPssrFrame> list = tPssrFrameService.selectTPssrFrameList(tPssrFrame);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrFrame> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/zdj.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 4;
            int num = 1;

            Row originalRow = sheet.getRow(4);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrFrame t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getBracketNumber());
                row.createCell(2).setCellValue(t.getPipelineNumber());
                row.createCell(3).setCellValue(t.getDevNo());
                row.createCell(4).setCellValue(t.getSupportType());
                row.createCell(5).setCellValue(t.getFramePipe());
                row.createCell(6).setCellValue(t.getFrameRoot());
                row.createCell(7).setCellValue(t.getFunctionalPart());
                row.createCell(8).setCellValue(t.getConnector());
                row.createCell(9);
                row.createCell(10);
                try {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(t.getConfirm1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirm2()));
                    String confirm1 = sysUser.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 9, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 10, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(11).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(12).setCellValue(t.getRemarks());


                //渲染样式
                for (int i = 0; i < 13; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename =  "PSSR_21_支（吊）架_" + UUID.randomUUID().toString() + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取支（吊）架详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrFrame item = tPssrFrameService.selectTPssrFrameById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), forShort));
        return AjaxResult.success(item);
    }

    /**
     * 新增支（吊）架
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:add')")
    @Log(title = "支（吊）架", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrFrame tPssrFrame) {
        tPssrFrame.setApproveStatus(0L);
        return toAjax(tPssrFrameService.insertTPssrFrame(tPssrFrame));
    }

    /**
     * 修改支（吊）架
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:edit')")
    @Log(title = "支（吊）架", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrFrame tPssrFrame) {
        tPssrFileService.updateFileRelevance(tPssrFrame.getFileIds(), forShort, tPssrFrame.getId(), tPssrFrame.getSubId());
        return toAjax(tPssrFrameService.updateTPssrFrame(tPssrFrame));
    }

    /**
     * 修改支（吊）架
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:edit')")
    @Log(title = "支（吊）架", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrFrame tPssrFrame) {
        return toAjax(tPssrFrameMapper.updateTPssrFrameByIds(tPssrFrame));
    }


    /**
     * 删除支（吊）架
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:remove')")
    @Log(title = "支（吊）架", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrFrameService.deleteTPssrFrameByIds(ids));
    }

    /**
     * 确认支（吊）架
     */
    @PreAuthorize("@ss.hasPermi('pssr:frame:edit')")
    @Log(title = "支（吊）架", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmFrame")
    public AjaxResult confirmFrame(@RequestBody TPssrFrame tPssrFrame) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        if (tPssrFrame.getTaskType() == 4) {
            //拆锁确认
            queryStatus = 1;
            approveStatus = 3;
            date = new Date();
        } else if (tPssrFrame.getTaskType() == 5) {
            //上锁确认
            queryStatus = 3;
            approveStatus = 2;
            date = new Date();
        }
        // 修改状态
        if (tPssrFrame.getIds() != null && tPssrFrame.getIds().length > 0) {
            for (Long id : tPssrFrame.getIds()) {
                TPssrFrame item = tPssrFrameService.selectTPssrFrameById(id);
                item.setApproveStatus(approveStatus);
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                tPssrFrameService.updateTPssrFrame(item);
            }
        } else {
            TPssrFrame lock = new TPssrFrame();
            lock.setSubId(tPssrFrame.getSubId());
            lock.setApproveStatus(queryStatus);
            for (TPssrFrame item : tPssrFrameService.selectTPssrFrameList(lock)) {
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                item.setApproveStatus(approveStatus);
                tPssrFrameService.updateTPssrFrame(item);
            }
        }
        //查询当前待审批的确认人
        TPssrFrame entity = new TPssrFrame();
        entity.setSubId(tPssrFrame.getSubId());
        entity.setApproveStatus(queryStatus);
        for (TPssrFrame item : tPssrFrameService.selectTPssrFrameList(entity)) {
            if (tPssrFrame.getTaskType() == 4) {
                if (item.getConfirm1().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            } else if (tPssrFrame.getTaskType() == 5) {
                if (item.getConfirm2().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }

        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrFrame.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();

    }

    /**
     * 驳回支（吊）架
     */
    @PutMapping("/turnDownFrame")
    public AjaxResult turnDownFrame(@RequestBody List<TPssrFrame> tPssrFrame) {
        if (CollectionUtils.isNotEmpty(tPssrFrame)) {
            String userId = getUserId().toString();
            Long subId = tPssrFrame.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrFrame item : tPssrFrame) {
                TPssrFrame blind = new TPssrFrame();
                blind.setId(item.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrFrameService.updateTPssrFrame(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(item.getSubId());
                turndown.setItemId(item.getId());
                turndown.setReason(item.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            ProcessEngine processEngine = ProcessEngines.getDefaultProcessEngine();
            TaskService taskService = processEngine.getTaskService();
            Task task = processEngine.getTaskService()//获取任务service
                    .createTaskQuery()//创建查询对象
                    .taskAssignee(userId)
                    .processInstanceId(approve.getProcessId()).singleResult();
            String taskId = task.getId();

            // 驳回 查询所有待审批的人员
            TPssrFrame blind = new TPssrFrame();
            blind.setSubId(subId);
            blind.setApproveStatus(1L);
            Set<String> installer = new HashSet<>();
            Set<String> remover = new HashSet<>();
            for (TPssrFrame item : tPssrFrameService.selectTPssrFrameList(blind)) {

                // 安装人员
                installer.add(item.getConfirm1());

                //拆除人员
                remover.add(item.getConfirm2());
            }


            //处理流程节点
            Map<String, Object> param = new HashMap<>();
            param.put("condition", 1);
            param.put("confirmUsers1", new ArrayList<>(installer));
            param.put("confirmUsers2", new ArrayList<>(remover));
            //认领任务
            processEngine.getTaskService().claim(taskId, userId);
            taskService.addComment(taskId, approve.getProcessId(), "驳回");
            taskService.complete(taskId, param);

            // 修改审批表和sub表
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
