package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.listener.pssr.ConfirmTaskCreateListener;
import com.ruoyi.project.pssr.domain.*;
import com.ruoyi.project.pssr.mapper.TPssrMeasureMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.impl.identity.Authentication;
import org.activiti.engine.runtime.ProcessInstance;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 人身防护Controller
 *
 * @author ssy
 * @date 2024-11-04
 */
@RestController
@RequestMapping("/pssr/measure")
public class TPssrMeasureController extends BaseController {
    @Autowired
    private ITPssrFileService tPssrFileService;
    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrMeasureService tPssrMeasureService;
    @Autowired
    private ITPssrProtectionService tPssrProtectionService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private HistoryService historyService;
    @Resource
    private TPssrMeasureMapper tPssrMeasureMapper;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "rsfh-m";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;

    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrMeasure> list = new ArrayList<TPssrMeasure>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrMeasure entity = new TPssrMeasure();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setDevNo(cellValue);
                    } else if (j == 1) {
                        entity.setDevName(cellValue);
                    } else if (j == 2) {
                        entity.setProMeasure(cellValue);
                    } else if (j == 3) {
                        entity.setNoDamage(cellValue);
                    } else if (j == 4) {
                        entity.setLocked(cellValue);
                    } else if (j == 5) {
                        entity.setNoWorn(cellValue);
                    } else if (j == 6) {
                        if (cellValue.length() > 3) {
                            entity.setConfirmationDate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));
                        }
                    } else if (j == 7) {
                        entity.setRemarks(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrMeasure t : list
        ) {
            failNum++;
            try {
                tPssrMeasureService.insertTPssrMeasure(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }

    /**
     * 查询人身防护列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrMeasure tPssrMeasure) {
        startPage();
        List<TPssrMeasure> list = tPssrMeasureService.selectTPssrMeasureList(tPssrMeasure);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "rsfh-m"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "rsfh-m"));
        });
        return getDataTable(list);
    }

    /**
     * 导出人身防护列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:export')")
    @Log(title = "人身防护", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrMeasure tPssrMeasure) {
        List<TPssrMeasure> list = tPssrMeasureService.selectTPssrMeasureList(tPssrMeasure);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrMeasure> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/rsfh2.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 4;
            int num = 1;

            Row originalRow = sheet.getRow(4);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrMeasure t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getDevNo());
                row.createCell(2).setCellValue(t.getDevName());
                row.createCell(3).setCellValue(t.getProMeasure());
                row.createCell(4).setCellValue(t.getNoDamage());
                row.createCell(5).setCellValue(t.getLocked());
                row.createCell(6).setCellValue(t.getNoWorn());
                row.createCell(7);
                row.createCell(8);
                try {
                    SysUser sysUser1 = sysUserService.selectUserById(Long.valueOf(t.getConfirm1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirm2()));
                    String confirm1 = sysUser1.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 7, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 8, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(9).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(10).setCellValue(t.getRemarks());

                //渲染样式
                for (int i = 0; i < 11; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename =  "PSSR_06_人身防护_现场转动设备防护措施检查表_" + UUID.randomUUID().toString() + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取人身防护详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrMeasure item = tPssrMeasureService.selectTPssrMeasureById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "rsfh-m"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "rsfh-m"));
        return AjaxResult.success(item);
    }

    /**
     * 新增人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:add')")
    @Log(title = "人身防护", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrMeasure tPssrMeasure) {
        tPssrMeasure.setApproveStatus(0L);
        tPssrMeasure.setCreaterCode(String.valueOf(getUserId()));
        tPssrMeasure.setCreatedate(new Date());
        return toAjax(tPssrMeasureService.insertTPssrMeasure(tPssrMeasure));
    }

    /**
     * 修改人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:edit')")
    @Log(title = "人身防护", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrMeasure tPssrMeasure) {
        tPssrFileService.updateFileRelevance(tPssrMeasure.getFileIds(), "rsfh-m", tPssrMeasure.getId(), tPssrMeasure.getSubId());
        return toAjax(tPssrMeasureService.updateTPssrMeasure(tPssrMeasure));
    }

    /**
     * 修改人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:edit')")
    @Log(title = "人身防护", businessType = BusinessType.UPDATE)
    @PutMapping("editBatch")
    public AjaxResult editBatch(@RequestBody TPssrMeasure tPssrMeasure) {
        return toAjax(tPssrMeasureMapper.updateTPssrMeasureByIds(tPssrMeasure));
    }

    /**
     * 删除人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:remove')")
    @Log(title = "人身防护", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrMeasureService.deleteTPssrMeasureByIds(ids));
    }

    /**
     * 确认人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:measure:edit')")
    @Log(title = "人身防护", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmMeasure")
    public AjaxResult confirmMeasure(@RequestBody TPssrMeasure tPssrMeasure) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        TPssrMeasure measure = new TPssrMeasure();
        if (tPssrMeasure.getTaskType() == 4) {
            //确认人1确认
            queryStatus = 1;
            approveStatus = 3;
            measure.setConfirm1(getUserId().toString());
            date = new Date();
        } else if (tPssrMeasure.getTaskType() == 5) {
            //确认人2确认
            queryStatus = 3;
            approveStatus = 2;
            measure.setConfirm2(getUserId().toString());
            date = new Date();
        }
        if (tPssrMeasure.getIds() != null && tPssrMeasure.getIds().length > 0) {
            for (Long id : tPssrMeasure.getIds()) {
                measure = tPssrMeasureService.selectTPssrMeasureById(id);
                if (measure.getConfirmationDate() == null && tPssrMeasure.getTaskType() == 5) {
                    measure.setConfirmationDate(new Date());
                }
                measure.setApproveStatus(approveStatus);
                measure.setUpdatedate(new Date());
                measure.setUpdaterCode(String.valueOf(getUserId()));
                tPssrMeasureService.updateTPssrMeasure(measure);
            }
        } else {
            measure.setSubId(tPssrMeasure.getSubId());
            measure.setApproveStatus(queryStatus);
            for (TPssrMeasure item : tPssrMeasureService.selectTPssrMeasureList(measure)) {
                if (tPssrMeasure.getTaskType() == 5) {
                    item.setConfirmationDate(new Date());
                }
                item.setApproveStatus(approveStatus);
                item.setUpdatedate(new Date());
                item.setUpdaterCode(String.valueOf(getUserId()));
                tPssrMeasureService.updateTPssrMeasure(item);
            }
        }
        //查询当前待审批的确认人
        TPssrProtection entity = new TPssrProtection();
        entity.setSubId(tPssrMeasure.getSubId());
        entity.setApproveStatus(queryStatus);
        TPssrProtection Protection = tPssrProtectionService.selectAllConfirmedPersonBySubId(entity);
        if (Protection != null) {
            //如果当前用户还有待审批任务
            if (tPssrMeasure.getTaskType() == 4 && StringUtils.isNotEmpty(Protection.getConfirm1())) {
                if (Protection.getConfirm1().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }

            if (tPssrMeasure.getTaskType() == 5 && StringUtils.isNotEmpty(Protection.getConfirm2())) {
                if (Protection.getConfirm2().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }
        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrMeasure.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();
    }


    /**
     * 驳回人身防护
     */
    @PutMapping("/turnDownMeasure")
    public AjaxResult turnDownMeasure(@RequestBody List<TPssrMeasure> tPssrMeasure) {
        if (CollectionUtils.isNotEmpty(tPssrMeasure)) {
            String userId = getUserId().toString();
            Long subId = tPssrMeasure.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrMeasure item : tPssrMeasure) {
                TPssrMeasure blind = new TPssrMeasure();
                blind.setId(item.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrMeasureService.updateTPssrMeasure(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(item.getSubId());
                turndown.setItemId(item.getId());
                turndown.setReason(item.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            try {
                runtimeService.deleteProcessInstance(approve.getProcessId(), "pssr2confirm");
                historyService.deleteHistoricProcessInstance(approve.getProcessId());
            } catch (Exception e) {
                logger.info("无运行时流程");
            }

            // 驳回 查询所有待审批的人员
            // 查询确认人
            TPssrProtection entity = new TPssrProtection();
            entity.setSubId(subId);
            entity.setApproveStatus(1L);
            TPssrProtection tPssrProtection = tPssrProtectionService.selectAllConfirmedPersonBySubId(entity);
            String confirmer1s = null;
            String confirmer2s = null;
            if (tPssrProtection != null) {
                confirmer1s = tPssrProtection.getConfirm1();
                confirmer2s = tPssrProtection.getConfirm2();
            }
            logger.info("=======================confirmer1s:{}", confirmer1s);
            logger.info("=======================confirmer2s:{}", confirmer2s);

            Set<String> confirmerUsers1 = new HashSet<>();
            Set<String> confirmerUsers2 = new HashSet<>();
            if (StringUtils.isNotEmpty(confirmer1s)) {
                confirmerUsers1.addAll(Arrays.asList(confirmer1s.split(",")));
            }
            if (StringUtils.isNotEmpty(confirmer2s)) {
                confirmerUsers2.addAll(Arrays.asList(confirmer2s.split(",")));
            }

            // 开始申请流程
            long businessKey = approve.getApproveId();
            //开始工作流、监听
            Authentication.setAuthenticatedUserId(userId);//设置当前申请人
            Map<String, Object> variables = new HashMap<>();
            variables.put("applyUser", userId);
            variables.put("confirmUsers1", new ArrayList<>(confirmerUsers1));
            variables.put("confirmUsers2", new ArrayList<>(confirmerUsers2));
            variables.put("confirmTaskCreateListener", new ConfirmTaskCreateListener());//发送邮件
            variables.put("chargePerson", approve.getSubCharge());
            //采用key来启动流程定义并设置流程变量，返回流程实例
            ProcessInstance pi = runtimeService.startProcessInstanceByKey("pssr2confirm", String.valueOf(businessKey), variables);

            // 修改审批表和sub表
            approve.setProcessId(pi.getProcessInstanceId());
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
