package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.listener.pssr.ConfirmTaskCreateListener;
import com.ruoyi.project.pssr.domain.TPssrApprove;
import com.ruoyi.project.pssr.domain.TPssrPipe;
import com.ruoyi.project.pssr.domain.TPssrSubcontent;
import com.ruoyi.project.pssr.domain.TPssrTurndown;
import com.ruoyi.project.pssr.mapper.TPssrPipeMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.sems.domain.TSpecdevYlgd;
import com.ruoyi.project.sems.mapper.TSpecdevYlgdMapper;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.impl.identity.Authentication;
import org.activiti.engine.runtime.ProcessInstance;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.*;

/**
 * 管道Controller
 *
 * @author ssy
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/pssr/pipe")
public class TPssrPipeController extends BaseController {
    @Autowired
    private ITPssrFileService tPssrFileService;

    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrPipeService tPssrPipeService;
    @Resource
    private TSpecdevYlgdMapper tSpecdevYlgdMapper;

    @Resource
    private TPssrPipeMapper tPssrPipeMapper;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private HistoryService historyService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "ylgd";

    /**
     * 查询管道列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrPipe tPssrPipe) {
        startPage();
        List<TPssrPipe> list = tPssrPipeService.selectTPssrPipeList(tPssrPipe);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "ylgd"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "ylgd"));
        });
        return getDataTable(list);
    }

    /**
     * 查询管道列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:add')")
    @GetMapping("/syncPipe")
    public AjaxResult syncPipe(TPssrPipe tPssrPipe) {
        TSpecdevYlgd tSpecdevYlgd = new TSpecdevYlgd();
        TPssrSubcontent subcontent = tPssrSubcontentService.selectTPssrSubcontentById(tPssrPipe.getSubId());
        List<TSpecdevYlgd> list = new ArrayList<>();
        //删除老数据
        tPssrPipeMapper.deleteTPssrPipeBySubId(tPssrPipe.getSubId());
        if (StringUtils.isNotEmpty(subcontent.getUnit())) {
            for (String unit : subcontent.getUnit().split(",")) {
                tSpecdevYlgd.setPlantCode("BCC");
                tSpecdevYlgd.setDevno("-" + unit);
                for (TSpecdevYlgd t : tSpecdevYlgdMapper.selectTSpecdevYlgdListForPssr(tSpecdevYlgd)) {
                    TPssrPipe pipe = new TPssrPipe();
                    pipe.setUnit(unit);
                    pipe.setPipelineName(t.getDevname());
                    pipe.setPipelineNo(t.getDevno());
                    pipe.setSubId(tPssrPipe.getSubId());

                    pipe.setChecked("√");
                    pipe.setRequire("√");
                    pipe.setValidity("√");

                    pipe.setApproveStatus(0L);
                    pipe.setCreatedate(new Date());
                    pipe.setCreaterCode(getUserId().toString());
                    tPssrPipeService.insertTPssrPipe(pipe);
                }
            }
        }

        return AjaxResult.success();
    }


    /**
     * 导出管道列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:export')")
    @Log(title = "管道", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrPipe tPssrPipe) {
        List<TPssrPipe> list = tPssrPipeService.selectTPssrPipeList(tPssrPipe);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrPipe> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/ylgd.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 4;
            int num = 1;

            Row originalRow = sheet.getRow(4);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrPipe t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getUnit());
                row.createCell(2).setCellValue(t.getPipelineName());
                row.createCell(3).setCellValue(t.getPipelineNo());
                row.createCell(4).setCellValue(t.getChecked());
                row.createCell(5).setCellValue(t.getRequire());
                row.createCell(6).setCellValue(t.getValidity());
                row.createCell(7);
                row.createCell(8);
                try {
                    SysUser sysUser1 = sysUserService.selectUserById(Long.valueOf(t.getConfirmer1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirmer2()));
                    String confirm1 =  sysUser1.getSignUrl();
                    String confirm2 =  sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 7, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 8, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(9).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(10).setCellValue(t.getRemarks());

                //渲染样式
                for (int i = 0; i < 11; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename =  "PSSR_22_压力管道_" + UUID.randomUUID().toString() + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取管道详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrPipe item = tPssrPipeService.selectTPssrPipeById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "ylgd"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "ylgd"));
        return AjaxResult.success(item);
    }

    /**
     * 新增管道
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:add')")
    @Log(title = "管道", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrPipe tPssrPipe) {

        tPssrPipe.setApproveStatus(0L);
        tPssrPipe.setCreatedate(new Date());
        tPssrPipe.setCreaterCode(getUserId().toString());
        return toAjax(tPssrPipeService.insertTPssrPipe(tPssrPipe));
    }

    /**
     * 修改管道
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:edit')")
    @Log(title = "管道", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrPipe tPssrPipe) {
        tPssrFileService.updateFileRelevance(tPssrPipe.getFileIds(), "ylgd", tPssrPipe.getId(), tPssrPipe.getSubId());
        return toAjax(tPssrPipeService.updateTPssrPipe(tPssrPipe));
    }

    /**
     * 修改管道
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:edit')")
    @Log(title = "管道批量修改", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editBatch(@RequestBody TPssrPipe tPssrPipe) {
        logger.info(JSON.toJSONString(tPssrPipe));
        return toAjax(tPssrPipeMapper.updateTPssrPipeByIds(tPssrPipe));
    }

    /**
     * 删除管道
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:remove')")
    @Log(title = "管道", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrPipeService.deleteTPssrPipeByIds(ids));
    }

    /**
     * 确认压力管道
     */
    @PreAuthorize("@ss.hasPermi('pssr:pipe:edit')")
    @Log(title = "确认压力管道", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmPipe")
    public AjaxResult confirmPipe(@RequestBody TPssrPipe tPssrPipe) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        TPssrPipe pipe = new TPssrPipe();
        if (tPssrPipe.getTaskType() == 4) {
            //确认人1确认
            queryStatus = 1;
            approveStatus = 3;
            pipe.setConfirmer1(getUserId().toString());
            date = new Date();
        } else if (tPssrPipe.getTaskType() == 5) {
            //确认人2确认
            queryStatus = 3;
            approveStatus = 2;
            pipe.setConfirmer2(getUserId().toString());
            date = new Date();
        }
        if (tPssrPipe.getIds() != null && tPssrPipe.getIds().length > 0) {
            for (Long id : tPssrPipe.getIds()) {
                pipe = tPssrPipeService.selectTPssrPipeById(id);
                if (queryStatus == 3) {
                    pipe.setConfirmationDate(new Date());
                }
                pipe.setApproveStatus(approveStatus);
                pipe.setUpdatedate(new Date());
                pipe.setUpdaterCode(String.valueOf(getUserId()));
                tPssrPipeService.updateTPssrPipe(pipe);
            }
        } else {
            pipe.setSubId(tPssrPipe.getSubId());
            pipe.setApproveStatus(queryStatus);
            for (TPssrPipe item : tPssrPipeService.selectTPssrPipeList(pipe)) {
                if (queryStatus == 3) {
                    item.setConfirmationDate(new Date());
                }
                item.setApproveStatus(approveStatus);
                item.setUpdatedate(new Date());
                item.setUpdaterCode(String.valueOf(getUserId()));
                tPssrPipeService.updateTPssrPipe(item);
            }
        }
        //查询当前待审批的确认人

        TPssrPipe entity = new TPssrPipe();
        entity.setSubId(tPssrPipe.getSubId());
        entity.setApproveStatus(queryStatus);
        TPssrPipe pssrPipe = tPssrPipeService.selectAllConfirmedPersonBySubId(entity);
        if (pssrPipe != null) {
            //如果当前用户还有待审批任务
            if (tPssrPipe.getTaskType() == 4 && StringUtils.isNotEmpty(pssrPipe.getConfirmer1())) {
                if (pssrPipe.getConfirmer1().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }

            if (tPssrPipe.getTaskType() == 5 && StringUtils.isNotEmpty(pssrPipe.getConfirmer2())) {
                if (pssrPipe.getConfirmer2().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }
        //无待审批任务结束当前用户流程
        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrPipe.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();
    }


    /**
     * 驳回压力管道
     */
    @PutMapping("/turnDownPipe")
    public AjaxResult turnDownPipe(@RequestBody List<TPssrPipe> tPssrPipe) {
        if (CollectionUtils.isNotEmpty(tPssrPipe)) {
            String userId = getUserId().toString();
            Long subId = tPssrPipe.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrPipe item : tPssrPipe) {
                TPssrPipe blind = new TPssrPipe();
                blind.setId(item.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrPipeService.updateTPssrPipe(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(item.getSubId());
                turndown.setItemId(item.getId());
                turndown.setReason(item.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            try {
                runtimeService.deleteProcessInstance(approve.getProcessId(), "pssr2confirm");
                historyService.deleteHistoricProcessInstance(approve.getProcessId());
            } catch (Exception e) {
                logger.info("无运行时流程");
            }

            // 驳回 查询所有待审批的人员

            //查询确认人
            TPssrPipe entity = new TPssrPipe();
            entity.setSubId(subId);
            entity.setApproveStatus(1L);
            TPssrPipe pssrPipe = tPssrPipeService.selectAllConfirmedPersonBySubId(entity);
            String confirmer1s = null;
            String confirmer2s = null;
            if (pssrPipe != null) {
                confirmer1s = pssrPipe.getConfirmer1();
                confirmer2s = pssrPipe.getConfirmer2();
            }
            logger.info("=======================confirmer1s:{}", confirmer1s);
            logger.info("=======================confirmer2s:{}", confirmer2s);

            Set<String> confirmerUsers1 = new HashSet<>();
            Set<String> confirmerUsers2 = new HashSet<>();
            if (StringUtils.isNotEmpty(confirmer1s)) {
                confirmerUsers1.addAll(Arrays.asList(confirmer1s.split(",")));
            }
            if (StringUtils.isNotEmpty(confirmer2s)) {
                confirmerUsers2.addAll(Arrays.asList(confirmer2s.split(",")));
            }

            // 开始申请流程
            long businessKey = approve.getApproveId();
            //开始工作流、监听
            Authentication.setAuthenticatedUserId(userId);//设置当前申请人
            Map<String, Object> variables = new HashMap<>();
            variables.put("applyUser", userId);
            variables.put("confirmUsers1", new ArrayList<>(confirmerUsers1));
            variables.put("confirmUsers2", new ArrayList<>(confirmerUsers2));
            variables.put("confirmTaskCreateListener", new ConfirmTaskCreateListener());//发送邮件
            variables.put("chargePerson", approve.getSubCharge());
            //采用key来启动流程定义并设置流程变量，返回流程实例
            ProcessInstance pi = runtimeService.startProcessInstanceByKey("pssr2confirm", String.valueOf(businessKey), variables);

            // 修改审批表和sub表
            approve.setProcessId(pi.getProcessInstanceId());
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
