package com.ruoyi.project.pssr.controller;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.pssr.domain.TPssrApprove;
import com.ruoyi.project.pssr.domain.TPssrPower;
import com.ruoyi.project.pssr.domain.TPssrSubcontent;
import com.ruoyi.project.pssr.domain.TPssrTurndown;
import com.ruoyi.project.pssr.mapper.TPssrPowerMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.ProcessEngine;
import org.activiti.engine.ProcessEngines;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.*;

/**
 * 临时电源Controller
 *
 * @author ssy
 * @date 2024-11-01
 */
@RestController
@RequestMapping("/pssr/power")
public class TPssrPowerController extends BaseController {
    @Resource
     private TPssrPowerMapper tPssrPowerMapper;
    @Autowired
    private ITPssrFileService tPssrFileService;

    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrPowerService tPssrPowerService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "lsdy";


    /**
     * 查询临时电源列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrPower tPssrPower) {
        startPage();
        List<TPssrPower> list = tPssrPowerService.selectTPssrPowerList(tPssrPower);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "lsdy"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "lsdy"));
        });
        return getDataTable(list);
    }

    /**
     * 导出临时电源列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:export')")
    @Log(title = "临时电源", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrPower tPssrPower) {
        List<TPssrPower> list = tPssrPowerService.selectTPssrPowerList(tPssrPower);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrPower> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/lsdy.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 3;
            int num = 1;

            Row originalRow = sheet.getRow(3);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrPower t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getCheckContent());
                row.createCell(2).setCellValue(t.getCheckResult());
                row.createCell(3);
                row.createCell(4);
                try {
                    SysUser sysUser = sysUserService.selectUserById(Long.valueOf(t.getConfirm1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirm2()));
                    String confirm1 = sysUser.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 3, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 4, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(5).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(6).setCellValue(t.getRemarks());


                //渲染样式
                for (int i = 0; i < 7; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename =  "PSSR_26_临时电源_" + UUID.randomUUID().toString() + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取临时电源详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrPower item = tPssrPowerService.selectTPssrPowerById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "lsdy"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "lsdy"));
        return AjaxResult.success(item);
    }

    /**
     * 新增临时电源
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:add')")
    @Log(title = "临时电源", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrPower tPssrPower) {
        tPssrPower.setApproveStatus(0L);
        return toAjax(tPssrPowerService.insertTPssrPower(tPssrPower));
    }

    /**
     * 修改临时电源
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:edit')")
    @Log(title = "临时电源", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrPower tPssrPower) {
        tPssrFileService.updateFileRelevance(tPssrPower.getFileIds(), "lsdy", tPssrPower.getId(), tPssrPower.getSubId());
        return toAjax(tPssrPowerService.updateTPssrPower(tPssrPower));
    }

    /**
     * 修改临时电源
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:edit')")
    @Log(title = "临时电源", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrPower tPssrPower) {
        return toAjax(tPssrPowerMapper.updateTPssrPowerByIds(tPssrPower));
    }
    /**
     * 删除临时电源
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:remove')")
    @Log(title = "临时电源", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrPowerService.deleteTPssrPowerByIds(ids));
    }

    /**
     * 确认临时电源
     */
    @PreAuthorize("@ss.hasPermi('pssr:power:edit')")
    @Log(title = "临时电源", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmPower")
    public AjaxResult confirmPower(@RequestBody TPssrPower tPssrPower) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        if (tPssrPower.getTaskType() == 4) {
            //拆锁确认
            queryStatus = 1;
            approveStatus = 3;
            date = new Date();
        } else if (tPssrPower.getTaskType() == 5) {
            //上锁确认
            queryStatus = 3;
            approveStatus = 2;
            date = new Date();
        }
        // 修改状态
        if (tPssrPower.getIds() != null && tPssrPower.getIds().length > 0) {
            for (Long id : tPssrPower.getIds()) {
                TPssrPower item = tPssrPowerService.selectTPssrPowerById(id);
                item.setApproveStatus(approveStatus);
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                tPssrPowerService.updateTPssrPower(item);
            }
        } else {
            TPssrPower lock = new TPssrPower();
            lock.setSubId(tPssrPower.getSubId());
            lock.setApproveStatus(queryStatus);
            for (TPssrPower item : tPssrPowerService.selectTPssrPowerList(lock)) {
                if (queryStatus == 3) {
                    item.setConfirmationDate(date);
                }
                item.setApproveStatus(approveStatus);
                tPssrPowerService.updateTPssrPower(item);
            }
        }
        //查询当前待审批的确认人
        TPssrPower entity = new TPssrPower();
        entity.setSubId(tPssrPower.getSubId());
        entity.setApproveStatus(queryStatus);
        for (TPssrPower item : tPssrPowerService.selectTPssrPowerList(entity)) {
            if (tPssrPower.getTaskType() == 4) {
                if (item.getConfirm1().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            } else if (tPssrPower.getTaskType() == 5) {
                if (item.getConfirm2().equals(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }

        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrPower.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();

    }

    /**
     * 驳回临时电源
     */
    @PutMapping("/turnDownPower")
    public AjaxResult turnDownPower(@RequestBody List<TPssrPower> tPssrPower) {
        if (CollectionUtils.isNotEmpty(tPssrPower)) {
            String userId = getUserId().toString();
            Long subId = tPssrPower.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrPower item : tPssrPower) {
                TPssrPower blind = new TPssrPower();
                blind.setId(item.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrPowerService.updateTPssrPower(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(item.getSubId());
                turndown.setItemId(item.getId());
                turndown.setReason(item.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            ProcessEngine processEngine = ProcessEngines.getDefaultProcessEngine();
            TaskService taskService = processEngine.getTaskService();
            Task task = processEngine.getTaskService()//获取任务service
                    .createTaskQuery()//创建查询对象
                    .taskAssignee(userId)
                    .processInstanceId(approve.getProcessId()).singleResult();
            String taskId = task.getId();

            // 驳回 查询所有待审批的人员
            TPssrPower blind = new TPssrPower();
            blind.setSubId(subId);
            blind.setApproveStatus(1L);
            Set<String> installer = new HashSet<>();
            Set<String> remover = new HashSet<>();
            for (TPssrPower item : tPssrPowerService.selectTPssrPowerList(blind)) {

                // 安装人员
                installer.add(item.getConfirm1());

                //拆除人员
                remover.add(item.getConfirm2());
            }


            //处理流程节点
            Map<String, Object> param = new HashMap<>();
            param.put("condition", 1);
            param.put("confirmUsers1", new ArrayList<>(installer));
            param.put("confirmUsers2", new ArrayList<>(remover));
            //认领任务
            processEngine.getTaskService().claim(taskId, userId);
            taskService.addComment(taskId, approve.getProcessId(), "驳回");
            taskService.complete(taskId, param);

            // 修改审批表和sub表
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
