package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.listener.pssr.ConfirmTaskCreateListener;
import com.ruoyi.project.pssr.domain.*;
import com.ruoyi.project.pssr.mapper.TPssrPumpOverhaulMapper;
import com.ruoyi.project.pssr.service.*;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.impl.identity.Authentication;
import org.activiti.engine.runtime.ProcessInstance;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 机泵检修完成确认清单Controller
 *
 * @author ssy
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/pssr/pumpOverhaul")
public class TPssrPumpOverhaulController extends BaseController {
    @Resource
     private TPssrPumpOverhaulMapper tPssrPumpOverhaulMapper;
    @Autowired
    private ITPssrFileService tPssrFileService;

    @Autowired
    private ITPssrTurndownService tPssrTurndownService;
    @Autowired
    private ITPssrPumpOverhaulService tPssrPumpOverhaulService;

    @Autowired
    private ITPssrPumpFillService tPssrPumpFillService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSubcontentService tPssrSubcontentService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private HistoryService historyService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "jb-o";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;

    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrPumpOverhaul> list = new ArrayList<TPssrPumpOverhaul>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrPumpOverhaul entity = new TPssrPumpOverhaul();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setUnit(cellValue);
                    } else if (j == 1) {
                        entity.setDevNo(cellValue);
                    } else if (j == 2) {
                        entity.setBackLoading(cellValue);
                    } else if (j == 3) {
                        entity.setOverhaulDone(cellValue);
                    } else if (j == 4) {
                        if (cellValue.length() > 3) {
                            entity.setConfirmationDate(new SimpleDateFormat(DateUtils.getDateFormat(cellValue)).parse(cellValue));
                        }
                    } else if (j == 5) {
                        entity.setRemarks(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrPumpOverhaul t : list
        ) {
            failNum++;
            try {
                tPssrPumpOverhaulService.insertTPssrPumpOverhaul(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }


    /**
     * 查询机泵检修完成确认清单列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrPumpOverhaul tPssrPumpOverhaul) {
        startPage();
        List<TPssrPumpOverhaul> list = tPssrPumpOverhaulService.selectTPssrPumpOverhaulList(tPssrPumpOverhaul);
        list.forEach(item -> {
            item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "jb-o"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "jb-o"));
        });
        return getDataTable(list);
    }

    /**
     * 导出机泵检修完成确认清单列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:export')")
    @Log(title = "机泵检修完成确认清单", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrPumpOverhaul tPssrPumpOverhaul) {
        List<TPssrPumpOverhaul> list = tPssrPumpOverhaulService.selectTPssrPumpOverhaulList(tPssrPumpOverhaul);
        return AjaxResult.success(exportTmpl(list));
    }

    public String exportTmpl(List<TPssrPumpOverhaul> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/jbjx.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 3;
            int num = 1;

            Row originalRow = sheet.getRow(3);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrPumpOverhaul t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getUnit());
                row.createCell(2).setCellValue(t.getDevNo());
                row.createCell(3).setCellValue(t.getBackLoading());
                row.createCell(4).setCellValue(t.getOverhaulDone());
                row.createCell(5);
                row.createCell(6);
                try {
                    SysUser sysUser1 = sysUserService.selectUserById(Long.valueOf(t.getConfirmer1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirmer2()));
                    String confirm1 =  sysUser1.getSignUrl();
                    String confirm2 =  sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 5, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 6, 1, 1);
                } catch (NumberFormatException e) {
                    throw new RuntimeException(e);
                }
                row.createCell(7).setCellValue(DateUtils.dateTime(t.getConfirmationDate()));
                row.createCell(8).setCellValue(t.getRemarks());

                //渲染样式
                for (int i = 0; i < 9; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename =  "PSSR_20_机泵_机泵检修完成确认清单_" + UUID.randomUUID().toString() + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取机泵检修完成确认清单详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrPumpOverhaul item = tPssrPumpOverhaulService.selectTPssrPumpOverhaulById(id);
        item.setFileList(tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), "jb-o"));
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "jb-o"));
        return AjaxResult.success(item);
    }

    /**
     * 新增机泵检修完成确认清单
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:add')")
    @Log(title = "机泵检修完成确认清单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrPumpOverhaul tPssrPumpOverhaul) {
        tPssrPumpOverhaul.setApproveStatus(0L);
        tPssrPumpOverhaul.setCreatedate(new Date());
        tPssrPumpOverhaul.setCreaterCode(String.valueOf(getUserId()));
        return toAjax(tPssrPumpOverhaulService.insertTPssrPumpOverhaul(tPssrPumpOverhaul));
    }

    /**
     * 修改机泵检修完成确认清单
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:edit')")
    @Log(title = "机泵检修完成确认清单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrPumpOverhaul tPssrPumpOverhaul) {
        tPssrFileService.updateFileRelevance(tPssrPumpOverhaul.getFileIds(), "jb-o", tPssrPumpOverhaul.getId(), tPssrPumpOverhaul.getSubId());
        return toAjax(tPssrPumpOverhaulService.updateTPssrPumpOverhaul(tPssrPumpOverhaul));
    }

    /**
     * 修改机泵检修完成确认清单
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:edit')")
    @Log(title = "机泵检修完成确认清单", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrPumpOverhaul tPssrPumpOverhaul) {
        return toAjax(tPssrPumpOverhaulMapper.updateTPssrPumpOverhaulByIds(tPssrPumpOverhaul));
    }

    /**
     * 删除机泵检修完成确认清单
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:remove')")
    @Log(title = "机泵检修完成确认清单", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrPumpOverhaulService.deleteTPssrPumpOverhaulByIds(ids));
    }

    /**
     * 确认人身防护
     */
    @PreAuthorize("@ss.hasPermi('pssr:pumpOverhaul:edit')")
    @Log(title = "人身防护", businessType = BusinessType.UPDATE)
    @PutMapping("/confirmPumpOverhaul")
    public AjaxResult confirmPumpOverhaul(@RequestBody TPssrPumpOverhaul tPssrPumpOverhaul) {
        long queryStatus = 0;
        long approveStatus = 0;
        Date date = null;
        TPssrPumpOverhaul pumpOverhaul = new TPssrPumpOverhaul();
        if (tPssrPumpOverhaul.getTaskType() == 4) {
            //确认人1确认
            queryStatus = 1;
            approveStatus = 3;
            pumpOverhaul.setConfirmer1(getUserId().toString());
            date = new Date();
        } else if (tPssrPumpOverhaul.getTaskType() == 5) {
            //确认人2确认
            queryStatus = 3;
            approveStatus = 2;
            pumpOverhaul.setConfirmer2(getUserId().toString());
            date = new Date();
        }
        if (tPssrPumpOverhaul.getIds() != null && tPssrPumpOverhaul.getIds().length > 0) {
            for (Long id : tPssrPumpOverhaul.getIds()) {
                pumpOverhaul = tPssrPumpOverhaulService.selectTPssrPumpOverhaulById(id);
                if (pumpOverhaul.getConfirmationDate() == null && tPssrPumpOverhaul.getTaskType() == 5) {
                    pumpOverhaul.setConfirmationDate(new Date());
                }
                pumpOverhaul.setApproveStatus(approveStatus);
                pumpOverhaul.setUpdatedate(new Date());
                pumpOverhaul.setUpdaterCode(String.valueOf(getUserId()));
                tPssrPumpOverhaulService.updateTPssrPumpOverhaul(pumpOverhaul);
            }
        } else {
            pumpOverhaul.setSubId(tPssrPumpOverhaul.getSubId());
            pumpOverhaul.setApproveStatus(queryStatus);
            for (TPssrPumpOverhaul item : tPssrPumpOverhaulService.selectTPssrPumpOverhaulList(pumpOverhaul)) {
                if (tPssrPumpOverhaul.getTaskType() == 5) {
                    item.setConfirmationDate(new Date());
                }
                item.setApproveStatus(approveStatus);
                item.setUpdatedate(new Date());
                item.setUpdaterCode(String.valueOf(getUserId()));
                tPssrPumpOverhaulService.updateTPssrPumpOverhaul(item);
            }
        }
        //查询当前待审批的确认人
        TPssrPumpFill entity = new TPssrPumpFill();
        entity.setSubId(tPssrPumpOverhaul.getSubId());
        entity.setApproveStatus(queryStatus);
        TPssrPumpFill queryData = tPssrPumpFillService.selectAllConfirmedPersonBySubId(entity);
        if (queryData != null) {
            //如果当前用户还有待审批任务
            if (tPssrPumpOverhaul.getTaskType() == 4 && StringUtils.isNotEmpty(queryData.getConfirmer1())) {
                if (queryData.getConfirmer1().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }

            if (tPssrPumpOverhaul.getTaskType() == 5 && StringUtils.isNotEmpty(queryData.getConfirmer2())) {
                if (queryData.getConfirmer2().contains(getUserId().toString())) {
                    return AjaxResult.success();
                }
            }
        }
        //无待审批任务结束当前用户流程

        // 因为流程关系所以approve一定会有且只有一条数据
        TPssrApprove tPssrApprove = tPssrApproveService.selectTPssrApproveBySubId(tPssrPumpOverhaul.getSubId());
        TPssrApproveController.handleConfirmApprove(tPssrApprove, getUserId().toString());
        return AjaxResult.success();
    }


    /**
     * 驳回人身防护
     */
    @PutMapping("/turnDownPumpOverhaul")
    public AjaxResult turnDownPumpOverhaul(@RequestBody List<TPssrPumpOverhaul> tPssrPumpOverhaul) {
        if (CollectionUtils.isNotEmpty(tPssrPumpOverhaul)) {
            String userId = getUserId().toString();
            Long subId = tPssrPumpOverhaul.get(0).getSubId();
            // 修改已选择数据的状态
            for (TPssrPumpOverhaul item : tPssrPumpOverhaul) {
                TPssrPumpOverhaul blind = new TPssrPumpOverhaul();
                blind.setId(item.getId());
                blind.setApproveStatus(1L);
                blind.setUpdatedate(new Date());
                blind.setUpdaterCode(getUserId().toString());
                tPssrPumpOverhaulService.updateTPssrPumpOverhaul(blind);
                // 新增驳回原因数据
                TPssrTurndown turndown = new TPssrTurndown();
                turndown.setForShort(forShort);
                turndown.setSubId(item.getSubId());
                turndown.setItemId(item.getId());
                turndown.setReason(item.getReason());
                turndown.setCreatedate(new Date());
                turndown.setCreaterCode(getUserId().toString());
                tPssrTurndownService.insertTPssrTurndown(turndown);
            }
            // 查询当前流程
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(subId);

            try {
                runtimeService.deleteProcessInstance(approve.getProcessId(), "pssr2confirm");
                historyService.deleteHistoricProcessInstance(approve.getProcessId());
            } catch (Exception e) {
                logger.info("无运行时流程");
            }

            // 驳回 查询所有待审批的人员
            // 查询确认人
            TPssrPumpFill entity = new TPssrPumpFill();
            entity.setSubId(subId);
            entity.setApproveStatus(1L);
            TPssrPumpFill queryData = tPssrPumpFillService.selectAllConfirmedPersonBySubId(entity);
            String confirmer1s = null;
            String confirmer2s = null;
            if (queryData != null) {
                confirmer1s = queryData.getConfirmer1();
                confirmer2s = queryData.getConfirmer2();
            }
            logger.info("=======================confirmer1s:{}", confirmer1s);
            logger.info("=======================confirmer2s:{}", confirmer2s);

            Set<String> confirmerUsers1 = new HashSet<>();
            Set<String> confirmerUsers2 = new HashSet<>();
            if (StringUtils.isNotEmpty(confirmer1s)) {
                confirmerUsers1.addAll(Arrays.asList(confirmer1s.split(",")));
            }
            if (StringUtils.isNotEmpty(confirmer2s)) {
                confirmerUsers2.addAll(Arrays.asList(confirmer2s.split(",")));
            }

            // 开始申请流程
            long businessKey = approve.getApproveId();
            //开始工作流、监听
            Authentication.setAuthenticatedUserId(userId);//设置当前申请人
            Map<String, Object> variables = new HashMap<>();
            variables.put("applyUser", userId);
            variables.put("confirmUsers1", new ArrayList<>(confirmerUsers1));
            variables.put("confirmUsers2", new ArrayList<>(confirmerUsers2));
            variables.put("confirmTaskCreateListener", new ConfirmTaskCreateListener());//发送邮件
            variables.put("chargePerson", approve.getSubCharge());
            //采用key来启动流程定义并设置流程变量，返回流程实例
            ProcessInstance pi = runtimeService.startProcessInstanceByKey("pssr2confirm", String.valueOf(businessKey), variables);

            // 修改审批表和sub表
            approve.setProcessId(pi.getProcessInstanceId());
            approve.setApproveStatus(1L);
            approve.setUpdatedate(new Date());
            approve.setUpdaterCode(getUserId().toString());
            tPssrApproveService.updateTPssrApprove(approve);

            TPssrSubcontent subcontent = new TPssrSubcontent();
            subcontent.setId(approve.getSubId());
            subcontent.setApproveStatus(1L);
            subcontent.setUpdatedate(new Date());
            subcontent.setUpdaterCode(getUserId().toString());
            tPssrSubcontentService.updateTPssrSubcontent(subcontent);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }
}
