package com.ruoyi.project.sems.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.sems.domain.TSpecCheck;
import com.ruoyi.project.sems.service.ITSpecCheckService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 特种设备检验记录Controller
 *
 * @author ruoyi
 * @date 2021-08-23
 */
@RestController
@RequestMapping("/sems/check")
public class TSpecCheckController extends BaseController
{
    @Autowired
    private ITSpecCheckService tSpecCheckService;

    /**
     * 查询特种设备检验记录列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TSpecCheck tSpecCheck)
    {
        startPage();
        List<TSpecCheck> list = tSpecCheckService.selectTSpecCheckList(tSpecCheck);
        return getDataTable(list);
    }

    /**
     * 导出特种设备检验记录列表
     */
    @Log(title = "特种设备检验记录", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TSpecCheck tSpecCheck)
    {
        List<TSpecCheck> list = tSpecCheckService.selectTSpecCheckList(tSpecCheck);
        ExcelUtil<TSpecCheck> util = new ExcelUtil<TSpecCheck>(TSpecCheck.class);
        return util.exportExcel(list, "check");
    }

    /**
     * 获取特种设备检验记录详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tSpecCheckService.selectTSpecCheckById(id));
    }

    /**
     * 新增特种设备检验记录
     */
    @Log(title = "特种设备检验记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TSpecCheck tSpecCheck)
    {
        return toAjax(tSpecCheckService.insertTSpecCheck(tSpecCheck));
    }

    /**
     * 修改特种设备检验记录
     */
    @Log(title = "特种设备检验记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TSpecCheck tSpecCheck)
    {
        return toAjax(tSpecCheckService.updateTSpecCheck(tSpecCheck));
    }

    /**
     * 删除特种设备检验记录
     */
    @Log(title = "特种设备检验记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tSpecCheckService.deleteTSpecCheckByIds(ids));
    }
}
