package com.ruoyi.project.training.spec.service;

import java.util.List;

import com.ruoyi.framework.security.LoginUser;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.training.spec.domain.TStFeedback;
import com.ruoyi.project.training.spec.domain.TStPlan;
import com.ruoyi.project.training.spec.domain.vo.TStPlanImportVO;

/**
 * 培训计划Service接口
 *
 * @author ruoyi
 * @date 2022-04-13
 */
public interface ITStPlanService
{
    /**
     * 查询季度培训计划列表
     * @param tStFeedback
     * @return
     */
    List<TStPlan> selectSeasonalTStPlanList(TStFeedback tStFeedback);

    /**
     * 查询月度培训计划列表
     * @param tStFeedback
     * @return
     */
    List<TStPlan> selectMonthlyTStPlanList(TStFeedback tStFeedback);

    /**
     * 查询培训计划
     *
     * @param id 培训计划ID
     * @return 培训计划
     */
    public TStPlan selectTStPlanById(Long id);

    /**
     * 查询培训计划列表
     *
     * @param tStPlan 培训计划
     * @return 培训计划集合
     */
    public List<TStPlan> selectTStPlanList(TStPlan tStPlan);

    /**
     * 新增培训计划
     *
     * @param tStPlan 培训计划
     * @return 结果
     */
    public int insertTStPlan(TStPlan tStPlan);

    /**
     * 修改培训计划
     *
     * @param tStPlan 培训计划
     * @return 结果
     */
    public int updateTStPlan(TStPlan tStPlan);

    /**
     * 批量删除培训计划
     *
     * @param ids 需要删除的培训计划ID
     * @return 结果
     */
    public int deleteTStPlanByIds(Long[] ids);

    /**
     * 删除培训计划信息
     *
     * @param id 培训计划ID
     * @return 结果
     */
    public int deleteTStPlanById(Long id);

    /**
     * 导入培养计划数据
     *
     * @param planList 培养计划数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param loginUser 操作用户
     * @param yearPlanId 年度培养计划id
     * @return 结果
     */
    String importData(List<TStPlanImportVO> planList, boolean isUpdateSupport, SysUser loginUser, Long yearPlanId);

    /**
     * 根据staffId修改“未完成”的继任者计划状态为“因离职中止”
     * @param staffId
     * @return
     */
    public int updateStudyStateByStaffId(Long staffId);
}
